#include "hCommandBlock.h"

void hAbstractCommand::clear(void){
	if(t!=NULL)
		delete t;
	t=NULL;
}

hAbstractCommand::hAbstractCommand(void){
	t=NULL;
}

hAbstractCommand::hAbstractCommand(hAbstractCommandBase * s){
	t=s;
}

hAbstractCommand::hAbstractCommand(const std::string & str){
	t=new hSingleCommand(str);
}

hAbstractCommand::hAbstractCommand(const hAbstractCommand &s){
	t=s.copy();
}

hAbstractCommand::~hAbstractCommand(void){
	clear();
}

const hAbstractCommand & hAbstractCommand::operator=(const hAbstractCommand &s){
	clear();
	t=s.copy();
}

const std::string & hAbstractCommand::getCommand(void){
	return t->getCommand();
}

unsigned long hAbstractCommand::getSize(void){
	return t->getSize();
}

bool hAbstractCommand::empty(void){
	return !(t->getSize());
}

void hAbstractCommand::pop(void){
	t->pop();
}

hAbstractCommand hAbstractCommand::front(void){
	return t->front();
}

unsigned char hAbstractCommand::getType(void){
	return t->getType();
}

hAbstractCommandBase * hAbstractCommand::copy(void) const{
	return t->copy();
}

hSingleCommand::hSingleCommand(const std::string &com){
	command=com;
	s=1;
}

unsigned long hSingleCommand::getSize(void){
	return s;
}

void hSingleCommand::pop(void){
	command.clear();
	s=0;
}

hAbstractCommand hSingleCommand::front(void){
	hSingleCommand *p=new hSingleCommand(*this);
	return p;
}

const std::string & hSingleCommand::getCommand(void){
	if(!s)
		throw hException("hSingleCommand::getCommand","FIFO underflow",hError);
	return command;
}

unsigned char hSingleCommand::getType(void){
	return single;
}

hAbstractCommandBase * hSingleCommand::copy(void) const{
	return new hSingleCommand(*this);
}

void hCommandBlock::push(const hCommandBlock &in){
	hCommandBlock *t=new hCommandBlock(in);
	hAbstractCommand t2(t);
	blocks.push_back(t2);
}

void hCommandBlock::push(const hSingleCommand &in){
	hSingleCommand *t=new hSingleCommand(in);
	hAbstractCommand t2(t);
	blocks.push_back(t2);
}

void hCommandBlock::push(const hAbstractCommand &in){
	blocks.push_back(in);
}

hAbstractCommand hCommandBlock::front(void){
	std::deque<hAbstractCommand>::iterator it;
	it=blocks.begin();
	return *it;
}

void hCommandBlock::pop(void){
	blocks.pop_front();
}

const std::string & hCommandBlock::getCommand(void){	
	if(blocks.size())
		return blocks[0].getCommand();
	else
		throw hException("hCommandBlock::getCommand","Command FIFO underflow",hError);
}

unsigned long hCommandBlock::getSize(void){
	return blocks.size();
}

unsigned char hCommandBlock::getType(void){
	return block;
}

hAbstractCommandBase * hCommandBlock::copy(void) const{
	return new hCommandBlock(*this);
}
