#ifndef HCOMMANDBLOCK_H
#define HCOMMANDBLOCK_H

#include <string>
#include <deque>

#include "hException.h"

class hAbstractCommandBase;

class hAbstractCommand{
	hAbstractCommandBase *t;
	void clear(void);
public:
	hAbstractCommand(void);
	hAbstractCommand(hAbstractCommandBase * s);
	hAbstractCommand(const hAbstractCommand &s);
	explicit hAbstractCommand(const std::string & str);
	~hAbstractCommand(void);
	const hAbstractCommand & operator=(const hAbstractCommand &s);
	const std::string & getCommand(void);
	unsigned long getSize(void);
	bool empty(void);
	void pop(void);
	hAbstractCommand front(void);
	unsigned char getType(void);
	hAbstractCommandBase * copy(void) const;
};


class hAbstractCommandBase{
public:
	virtual const std::string & getCommand(void)=0;
	virtual unsigned long getSize(void)=0;
	virtual void pop(void)=0;
	virtual hAbstractCommand front(void)=0;
	virtual unsigned char getType(void)=0;
	virtual hAbstractCommandBase * copy(void) const=0;
};

class hSingleCommand:public hAbstractCommandBase{
	std::string command;
	char s;
public:
	hSingleCommand(const std::string &com);
	const std::string & getCommand(void);
	unsigned long getSize(void);
	void pop(void);
	hAbstractCommand front(void);
	unsigned char getType(void);
	hAbstractCommandBase * copy(void) const;
};


//This class implements a fifo of commands
//`commands' can be either a single command represented by hSimpleCommand
//or a block of commands represented by hCommandBlock
class hCommandBlock:public hAbstractCommandBase{
	std::deque<hAbstractCommand> blocks;
public:
	hCommandBlock(void){};
	//Insert a block at the back of the fifo
	void push(const hCommandBlock &in);
	//insert a single command at the back of the fifo
	void push(const hSingleCommand &in);
	void push(const hAbstractCommand &in);
	//Access the fisrt element at the front of the fifo
	hAbstractCommand front(void);
	//delete the element at the front of the fifo
	void pop(void);
	hAbstractCommandBase * copy(void) const;
	unsigned long getSize(void);
	unsigned char getType(void);
	//dont use. throws exception
	const std::string & getCommand(void);
};


enum blockType{
	single=0,
	block
};

#endif
