/***************************************************************************
 *   Copyright (C) 2007 by Christos Gkekas                                 *
 *   chggr@ee.auth.gr                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef HCOMMANDHISTORY_H
#define HCOMMANDHISTORY_H

#include <vector>
#include <iostream>
#include <fstream>
#include <stdlib.h>
#include <time.h>
#include <string>
#include <dirent.h>
#include <sstream>
#include "hGlobals.h"

using namespace std;

class hCommandHistory
{
  public:
    //Constructor and destructor.
    hCommandHistory(); //Constructs a hCommandHistory object.

    //Sets the number of days or sessions history files will be kept.
    void setNumDays(int days);
    void setNumSessions(int sessions);

    //Clears history files that are not needed anymore.
    void clearHistoryFiles();

    //Delete all history files.
    void deleteHistory();

    //Reads the history data from files.
    void readHistoryData();

    //These functions are used to create a new session, add commands to it and finally save them in the corresponding file.
    void createCurrentSession();
    void addCommand(string command);
    void saveCurrentSession();

    //This function returns the history as a vector of strings.
    vector<string> getHistoryData();

    //Prints the history contents at std::out, placing numbers before each command so that it can be called again using the "getHistoryCommand" function.
    void printHistoryData();
    string getHistoryCommand(int location);


  private:
    //The number of days or number of sessions that history files will be kept.
    int numDays;
    int numSessions;

    //A boolean value indicating whether a new session was created.
    bool currentSessionCreated;

    //A string vector holding the history data.
    vector<string> historyData;

    //The name of the new file which is going to be created upon exit.
    string newFileName;

    //A function used to read the directory ~/.englab/history and report its contents sorted.
    vector<string> readHistoryDirectory();
};

#endif

