/***************************************************************************
 *   Copyright (C) 2007 by poumpa                                          *
 *   poumpa@users.sf.net                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef HCOMMANDPARSER_H
#define HCOMMANDPARSER_H

#include <iostream>
#include <string>
using std::string;

#include "hVariableTree.h"
#include "hVariable.h"
#include "hExpressionParser.h"
#include "hGlobals.h"


#define STRING_INDEX std::string::size_type

class hCommandParser
{
	hVariableTree		*memory;
	hFunction 	*functiontree;
	bool				display;
	hExpressionParser	exprParser;

	unsigned long		m_dim, *m_size;

	// utility function that removes all the spaces (' ') in the string
	void removeSpaces(string & s);

	// utility function that removes the spaces in the edges of the string
	void removeEdgeSpaces(string & s);

	void setCmdAnswer(const hVariable & var);

	// create a variable of "type" with "name" and default values
	hVariable createDefaultVariable(const string & type,const string & name,
											unsigned int dim,unsigned long *size);

	// create a variable that is equal to "var" and is of "type" (doing
	// the necessary conversions)
	hVariable createVariableFromValues(const string & type,const string & name,const hVariable & var);

	// process the command when there are no registered words
	hVariable processNoRegWord(string & cmd,hFunction *functions);

	// process the command when there are registered words
	// "type_idx" is the index to the registered word
	hVariable processRegWord(const string & cmd,unsigned int type_idx,hFunction *functions);

	// find a valid assignment operator '='
	STRING_INDEX findAssignment(const string &cmd);

	// validation functions
	bool isNameValid(const string & name);
	bool isNumber(char c);
	bool isSymbol(char c);
	bool isArgValid(hVariable & arg);

	// checks if the expression has spaces between invalid variables
	bool isValidExpression(const string & expr);

	// check if the dimensions and sizes are the same
	bool checkSize(const unsigned int &dim1,const unsigned long &dim2,const unsigned long * const size1,const std::vector<unsigned int> size2);
	bool checkSize(const unsigned int &dim1,const unsigned long &dim2,const vector<unsigned int> size1,const unsigned long * const size2);
	bool checkSize(const unsigned int &dim1,const unsigned long &dim2,const vector<unsigned int> size1,const vector<unsigned int> size2);

	// check if the index is appropriate for the assignment
	bool checkIdx(const unsigned int &var_dim,const unsigned long &idx_dim,const vector<unsigned int> var_size,const unsigned long * const idx);


	//bool isSimpleVar(const unsigned int &dim,const unsigned int * const var_size);
	bool isSimpleVar(hVariable &var);

	// commandparser instances CANNOT be copied
// 	hCommandParser(const hCommandParser &) {}
// 	hCommandParser & operator=(const hCommandParser &) {}

public:
	static const int reg_words_count = 30;
	static const string registered_words[];
	static const string invalid_chars;
	static const string className;

	//hCommandParser(){};
	hCommandParser(hVariableTree *m=NULL) : memory(m) {};
	~hCommandParser() {  };

	// main usage function
	hVariable parseCommand(string cmd,hVariableTree *mem,hFunction *functions);

	// set the memory (variable tree)
	void setMemory(hVariableTree *m) { memory=m; }
	bool respond(void) { return display; }
	string lastName;
	std::vector<std::string> multipleArgs(std::string cmd,std::string regWord);
	void checkForInvalidSpaces(std::string command);
};

#endif
