/***************************************************************************
 *   Copyright (C) 2007 by manos,Harry Serenis                             *
 *   etsardou@users.sf.net,bobomastoras@users.sf.net                       * 
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "hConversions.h"


hConversions::hConversions(void){
	name="conversions";
	functionNames["int"]=&ConversionsToint;
	functionNames["uint"]=&ConversionsTouint;
	functionNames["uchar"]=&ConversionsTouchar;
	functionNames["char"]=&ConversionsTochar;
	functionNames["usint"]=&ConversionsTousint;
	functionNames["float"]=&ConversionsTofloat;
	functionNames["double"]=&ConversionsTodouble;
	functionNames["ldouble"]=&ConversionsToldouble;
	functionNames["complex"]=&ConversionsTocomplex;
	functionNames["dcomplex"]=&ConversionsTodcomplex;
	functionNames["ldcomplex"]=&ConversionsToldcomplex;
}

std::map<std::string,hAbstractFunction *> hConversions::getNames(void){
	return functionNames;
}

std::string hConversions::toolBoxName(void){
	return name;
}


hStructure hToint::execute(hStructure &arg){
	hStructure ret;
	if(arg.size()==0)
		throw hException("hInt::execute","Cannot calculate function with zero input arguments",hError);
	ret.reserve(arg.size());
	ret.push_back(arg[0].cast(hsInt));
	return ret;
}
std::string hToint::getHelp(void)
{
	return "Syntax: int(in)\n\nDescription: Function 'int', converts it's input variable to a variable of type int. The input variable remains unchanged\n\nInput parameters: 'in' can be a variable of any dimensions and any type, except for complex.\n\nExceptions: hException::hError if input argument's number is 0.\n\nReturns: a variable with the same dimensions as 'in', and of type int.\n\n"; 
}

hStructure hTouint::execute(hStructure &arg){
	hStructure ret;
	if(arg.size()==0)
		throw hException("hUint::execute","Cannot calculate function with zero input arguments",hError);
	ret.reserve(arg.size());
	ret.push_back(arg[0].cast(huInt));
	return ret;
}
std::string hTouint::getHelp(void)
{
	return "Syntax: uint(in)\n\nDescription: Function 'uint', converts it's input variable to a variable of type unsigned integer. The input variable remains unchanged\n\nInput parameters: 'in' can be a variable of any dimensions and any type,except complex.\n\nExceptions: hException::hError if input argument's number is 0.\n\nReturns: a variable with the same dimensions as 'in', and of type unsigned integer .\n\n" ;
}
hStructure hTouchar::execute(hStructure &arg){
	hStructure ret;
	if(arg.size()==0)
		throw hException("hUchar::execute","Cannot calculate function with zero input arguments",hError);
	ret.reserve(arg.size());
	ret.push_back(arg[0].cast(huChar));
	return ret;
}
std::string hTouchar::getHelp(void)
{
	return "Syntax: uchar(in)\n\nDescription: Function 'uchar', converts it's input variable to a variable of type unsigned character. The input variable remains unchanged\n\nInput parameters: 'in' can be a variable of any dimensions and any type,except complex.\n\nExceptions: hException::hError if input argument's number is 0.\n\nReturns: a variable with the same dimensions as 'in', and of type unsigned character .\n\n" ;
}
hStructure hTochar::execute(hStructure &arg){
	hStructure ret;
	if(arg.size()==0)
		throw hException("hChar::execute","Cannot calculate function with zero input arguments",hError);
	ret.reserve(arg.size());
	ret.push_back(arg[0].cast(hsChar));
	return ret;
}
std::string hTochar::getHelp(void)
{
	return "Syntax: char(in)\n\nDescription: Function 'char', converts it's input variable to a variable of type signed character. The input variable remains unchanged.\n\nInput parameters: 'in' can be a variable of any dimensions and any type, except for complex.\n\nExceptions: hException::hError if input argument's number is 0.\n\nReturns: a variable with the same dimensions as 'in', and of type unsigned char.\n\n" ;
}

hStructure hTousint::execute(hStructure &arg){
	hStructure ret;
	if(arg.size()==0)
		throw hException("hSint::execute","Cannot calculate function with zero input arguments",hError);
	ret.reserve(arg.size());
	ret.push_back(arg[0].cast(huShortInt));
	return ret;
}
std::string hTousint::getHelp(void)
{
	return "Syntax: usint(in)\n\nDescription: Function 'usint', converts it's input variable to a variable of type unsigned short integer. The input variable remains unchanged\n\nInput parameters: 'in' can be a variable of any dimensions and any type,except complex.\n\nExceptions: hException::hError if input argument's number is 0.\n\nReturns: a variable with the same dimensions as 'in', and of type unsigned short integer .\n\n" ;
}
hStructure hTofloat::execute(hStructure &arg){
	hStructure ret;
	if(arg.size()==0)
		throw hException("hFloat::execute","Cannot calculate function with zero input arguments",hError);
	ret.reserve(arg.size());
	ret.push_back(arg[0].cast(hFloat));
	return ret;
}
std::string hTofloat::getHelp(void)
{
	return "Syntax: float(in)\n\nDescription: Function 'float', converts it's input variable to a variable of type float. The input variable remains unchanged\n\nInput parameters: 'in' can be a variable of any dimensions and any type, except for complex.\n\nExceptions: hException::hError if input argument's number is 0.\n\nReturns: a variable with the same dimensions as 'in', and of type float.\n\n" ;
}
hStructure hTodouble::execute(hStructure &arg){
	hStructure ret;
	if(arg.size()==0)
		throw hException("hDouble::execute","Cannot calculate function with zero input arguments",hError);
	ret.reserve(arg.size());
	ret.push_back(arg[0].cast(hDouble));
	return ret;
}
std::string hTodouble::getHelp(void)
{
	return "Syntax: double(in)\n\nDescription: Function 'double', converts it's input variable to a variable of type double. The input variable remains unchanged\n\nInput parameters: 'in' can be a variable of any dimensions and any type, except for complex.\n\nExceptions: hException::hError if input argument's number is 0.\n\nReturns: a variable with the same dimensions as 'in', and of type double.\n\n" ;
}
hStructure hToldouble::execute(hStructure &arg){
	hStructure ret;
	if(arg.size()==0)
		throw hException("hLdouble::execute","Cannot calculate function with zero input arguments",hError);
	ret.reserve(arg.size());
	ret.push_back(arg[0].cast(hLDouble));
	return ret;
}
std::string hToldouble::getHelp(void)
{
	return "Syntax: ldouble(in)\n\nDescription: Function 'ldouble', converts it's input variable to a variable of type long double . The input variable remains unchanged\n\nInput parameters: 'in' can be a variable of any dimensions and any type,except complex.\n\nExceptions: hException::hError if input argument's number is 0.\n\nReturns: a variable with the same dimensions as 'in', and of type long double .\n\n" ;
}
hStructure hTocomplex::execute(hStructure &arg){
	hStructure ret;
	if(arg.size()==0)
		throw hException("hComplex::execute","Cannot calculate function with zero input arguments",hError);
	ret.reserve(arg.size());
	ret.push_back(arg[0].cast(hComplex));
	return ret;
}
std::string hTocomplex::getHelp(void)
{
	return "Syntax: complex(in)\n\nDescription: Function 'complex', converts it's input variable to a variable of type complex (float). The input variable remains unchanged\n\nInput parameters: 'in' can be a variable of any dimensions and any type.\n\nExceptions: hException::hError if input argument's number is 0.\n\nReturns: a variable with the same dimensions as 'in', and of type complex (float).\n\n" ;
}
hStructure hTodcomplex::execute(hStructure &arg){
	hStructure ret;
	if(arg.size()==0)
		throw hException("hDcomplex::execute","Cannot calculate function with zero input arguments",hError);
	ret.reserve(arg.size());
	ret.push_back(arg[0].cast(hDComplex));
	return ret;
}
std::string hTodcomplex::getHelp(void)
{
	return "Syntax: dcomplex(in)\n\nDescription: Function 'dcomplex', converts it's input variable to a variable of type complex (double). The input variable remains unchanged\n\nInput parameters: 'in' can be a variable of any dimensions and any type.\n\nExceptions: hException::hError if input argument's number is 0.\n\nReturns: a variable with the same dimensions as 'in', and of type complex (double).\n\n" ;
}
hStructure hToldcomplex::execute(hStructure &arg){
	hStructure ret;
	if(arg.size()==0)
		throw hException("hLdcomplex::execute","Cannot calculate function with zero input arguments",hError);
	ret.reserve(arg.size());
	ret.push_back(arg[0].cast(hLDComplex));
	return ret;
}
std::string hToldcomplex::getHelp(void)
{
	return "Syntax: ldcomplex(in)\n\nDescription: Function 'ldcomplex', converts it's input variable to a variable of type long double complex. The input variable remains unchanged\n\nInput parameters: 'in' can be a variable of any dimensions and any type.\n\nExceptions: hException::hError if input argument's number is 0.\n\nReturns: a variable with the same dimensions as 'in', and of type long double complex.\n\n"; 
}

