/***************************************************************************
 *   Copyright (C) 2007 by manos,Harry Serenis                             *
 *   etsardou@users.sf.net,bobomastoras@users.sf.net                       * 
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef HCONVERSIONS_H
#define HCONVERSIONS_H



/**@file hConversions.h
In order to compile an englab shared library you need to include hFunctionHeaders.h. hFunctionHeaders.h depends on hVariableHeaders.h, hException.h, hVarTempExcep.h. So you actually need 4 headers files.
@see hFunctionHeaders.h
**/
#include "hFunctionHeaders.h"
#include "hVarType.h"


/**@file hConversions.h
First you need to declare the classes that implement the functions. Each class can implement only one function
**/

/**@defgroup functionClasses Function Classes Implementation
@brief Function classes implement the actual functions

Function classes publicly inherit from class hAbstractFunction witch is pure abstract. So you have to implement methods: getName, getNumberofArgs, getNumberOfOut and execute. The first three are prety straight forward. Method execute is the actual function implementation. It is called each time the users refers to a function with name equal to getName(). Class names are indifferent.
**/
/**@{
@brief Simple function class than implements the sinus function

This class publicly inherits from class hAbstractFunction and implements all of it's pure virtual methods.
*/
class hToint:public hAbstractFunction{
	/**@brief Number of input and output arguments

	private variable members inArgs and outArgs are not mantatory but are used here to have a more general controll to the number of input and output arguments. Members are declared static and constant since they don't depend on object declaration and don't change during code execution
	*/
	static const unsigned short inArgs=1,outArgs=1;
public:
	/**
	@brief Executes the actual function code on input arguments and returns the result

	Method must be implemented since class inherits from pure abstract class hAbstractFunction
	@param arg is a container of type hStructure. User must implement code in order to check for valid number of input arguments and in the case of an error he must thow an hException
	@return an hStructure container that stores all output data. User must make sure that the size the container is equal to outArgs
	@see hFunctionHeaders.h
	@see hAbstractFunction
	@see hStructure
	@see hException
	*/
	hStructure execute(hStructure &arg);
	///Returns the name of the function that the class implements
	std::string getName(void){return "int";}
	///Returns the number of input arguments
	unsigned short getNumberOfArgs(void){return inArgs;}
	///Returns the number of output arguments
	unsigned short getNumberOfOut(void){return outArgs;}
	std::string getHelp(void);
};

class hTouint:public hAbstractFunction{
	static const unsigned short inArgs=1,outArgs=1;
public:
	hStructure execute(hStructure &arg);
	std::string getName(void){return "uint";}
	unsigned short getNumberOfArgs(void){return inArgs;}
	unsigned short getNumberOfOut(void){return outArgs;}
	std::string getHelp(void);
};
class hTouchar:public hAbstractFunction{
	static const unsigned short inArgs=1,outArgs=1;
public:
	hStructure execute(hStructure &arg);
	std::string getName(void){return "uchar";}
	unsigned short getNumberOfArgs(void){return inArgs;}
	unsigned short getNumberOfOut(void){return outArgs;}
	std::string getHelp(void);
};
class hTochar:public hAbstractFunction{
	static const unsigned short inArgs=1,outArgs=1;
public:
	hStructure execute(hStructure &arg);
	std::string getName(void){return "char";}
	unsigned short getNumberOfArgs(void){return inArgs;}
	unsigned short getNumberOfOut(void){return outArgs;}
	std::string getHelp(void);
};
class hTousint:public hAbstractFunction{
	static const unsigned short inArgs=1,outArgs=1;
public:
	hStructure execute(hStructure &arg);
	std::string getName(void){return "usint";}
	unsigned short getNumberOfArgs(void){return inArgs;}
	unsigned short getNumberOfOut(void){return outArgs;}
	std::string getHelp(void);
};
class hTofloat:public hAbstractFunction{
	static const unsigned short inArgs=1,outArgs=1;
public:
	hStructure execute(hStructure &arg);
	std::string getName(void){return "float";}
	unsigned short getNumberOfArgs(void){return inArgs;}
	unsigned short getNumberOfOut(void){return outArgs;}
	std::string getHelp(void);
};
class hTodouble:public hAbstractFunction{
	static const unsigned short inArgs=1,outArgs=1;
public:
	hStructure execute(hStructure &arg);
	std::string getName(void){return "double";}
	unsigned short getNumberOfArgs(void){return inArgs;}
	unsigned short getNumberOfOut(void){return outArgs;}
	std::string getHelp(void);
};
class hToldouble:public hAbstractFunction{
	static const unsigned short inArgs=1,outArgs=1;
public:
	hStructure execute(hStructure &arg);
	std::string getName(void){return "ldouble";}
	unsigned short getNumberOfArgs(void){return inArgs;}
	unsigned short getNumberOfOut(void){return outArgs;}
	std::string getHelp(void);
};
class hTocomplex:public hAbstractFunction{
	static const unsigned short inArgs=1,outArgs=1;
public:
	hStructure execute(hStructure &arg);
	std::string getName(void){return "complex";}
	unsigned short getNumberOfArgs(void){return inArgs;}
	unsigned short getNumberOfOut(void){return outArgs;}
	std::string getHelp(void);
};
class hTodcomplex:public hAbstractFunction{
	static const unsigned short inArgs=1,outArgs=1;
public:
	hStructure execute(hStructure &arg);
	std::string getName(void){return "dcomplex";}
	unsigned short getNumberOfArgs(void){return inArgs;}
	unsigned short getNumberOfOut(void){return outArgs;}
	std::string getHelp(void);
};
class hToldcomplex:public hAbstractFunction{
	static const unsigned short inArgs=1,outArgs=1;
public:
	hStructure execute(hStructure &arg);
	std::string getName(void){return "ldcomplex";}
	unsigned short getNumberOfArgs(void){return inArgs;}
	unsigned short getNumberOfOut(void){return outArgs;}
	std::string getHelp(void);
};

//@}

/**@defgroup packageClass Package Main Class Implementation
@brief After all function classes have been defined, a package class must be defined.

This Class must contain a single object of each function class, since in order to use an execute method an object must have been declared. A map between function names an pointers to function objects must also exist. This map is loaded by the package class's constructor. The only public methods of the package class are the constructor and a function returning the class's map. Finaly an object of this class must be declared in order to actually allocate memory for all objects and gain access to the execute methods.
*/
//@{
/**
@brief This is a simple package class for the hMath package.

Only functions sin, cos, tan have been defined using function classes hSin, hCos, hTan respectivly. A map between function names and function classes also exists as needed
**/
class hConversions:public hToolBox{
	//std::map<std::string,hAbstractFunction *> functionNames;
	hToint ConversionsToint;
	hTouint ConversionsTouint;
	hTousint ConversionsTousint;
	hTouchar ConversionsTouchar;
	hTochar ConversionsTochar;
	hTofloat ConversionsTofloat;
	hTodouble ConversionsTodouble;
	hToldouble ConversionsToldouble;
	hTocomplex ConversionsTocomplex;
	hTodcomplex ConversionsTodcomplex;
	hToldcomplex ConversionsToldcomplex;
public:
	hConversions(void);
	std::map<std::string,hAbstractFunction *> getNames(void);
	std::string toolBoxName(void);
};
//@}

///@defgroup loader Shared Objected Loader Function
/**@brief Returns pointers to objects implementing functions

This class is required in order for the main program to register all new functions. It is the only interface between the static program and the shared object so it is abselutely mandatory. All other classes, methods, variables and functions can be implemented differently than discribed, exept from this one. Function prototype must be declared using the <i>extern "C"</i> modifier. This canseles C++ name mangling in object files, C naming is used instead. This enables ld dynamic object loaded to find the apropriate symbols. 
*/



#endif
