/***************************************************************************
 *   Copyright (C) 2007 by Harry Serenis                                   *
 *   bobomastoras@users.sf.net                                             * 
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
//#include <hException.h>
//#include <hGlobals.h>

#include "hFunction.h"

void hFunction::insertFunction(const std::string &name, hAbstractFunction *func)
{
	data.insert(std::make_pair(name,func));
	func->setExec(this);
}

void hFunction::loadToolBox(hToolBox *box){
	boxNames.push_back(box);
	std::map<std::string,hAbstractFunction *> functions;
	std::map<std::string,hAbstractFunction *>::iterator it;
	for(it=functions.begin();it!=functions.end();++it)
		it->second->setExec(this);
	insertFunction(box->getNames());
}

std::vector<std::string> hFunction::getToolBoxFunctions(std::string name){
	std::list<hToolBox *>::iterator it;
//	it=data.find(name);
	for(it=boxNames.begin();it!=boxNames.end();++it){
		hToolBox *temp;
		temp=*it;
		if(!(temp->toolBoxName()).compare(name)){
			return (*it)->getNamesOnly();
		}
	}
	throw hException("hFunction::getToolBoxFunctions","ToolBox doesn't exist",hError);
}


std::string hFunction::getHelp(std::vector<std::string> argv){
	hToolBox *temp2;
	if(!argv.size())
		throw hException("hFunction::getHelp","Invalid input arguments",hError);
	std::list<hToolBox *>::iterator it;
	for(it=boxNames.begin();it!=boxNames.end();++it){
		temp2=*it;
		if(!temp2->toolBoxName().compare(argv[0]))
			break;
	}
	if(it==boxNames.end())
		throw hException("hFunction::getHelp","Toolbox not found",hError);
	std::map<std::string,hAbstractFunction *> temp;
	std::map<std::string,hAbstractFunction *>::iterator fit;
	temp=(*it)->getNames();
	fit=temp.find(argv[1]);
	if(fit!=temp.end())
		return fit->second->getHelp();
	else
		throw hException("hFunction::getHelp","Function `"+argv[1]+"' doen't exist in `"+argv[0]+"' toolbox",hError);
//	return temp[argv[1]]->getHelp();
}

/*void hFunction::removeFunction(const std::string &name)
{
	data.erase(name);
}*/

/*AbsFuncVector hFunction::getFunction(const std::string &name)

{

	std::pair<functionTable::iterator, functionTable::iterator> bounds;

	bounds = data.equal_range(name);



	if (bounds.first == bounds.second)

	{

		// the function doesn't exist, return empty vector

		return AbsFuncVector();

	}



	AbsFuncVector validFuncs(std::distance(bounds.first,bounds.second));

	functionTable::iterator i;

	for (i=bounds.first ; i!=bounds.second ; ++i)

	{

		validFuncs.push_back(i->second);

	}



	return validFuncs;

}*/

hStructure hFunction::execute(std::string name,hStructure &args){
	functionTable::iterator it;
	it=data.find(name);
	if(it!=data.end()){
		try{
			return data.find(name)->second->execute(args);
		}catch(hException &e){
			throw hException("hFunction::execute","Function error",hError);
		}catch(std::exception &e){
			std::string temp("Unexpected exception: ");
			temp+=std::string(e.what());
			throw hException("hFunction::execute",temp,hError);
		}
	}else{
		std::string err("Function '");
		err+=name;
		err+="' doesn't exist\n";
		throw hException("hFunction::execute",err,hInvalidFunction);
	}
}

void hFunction::insertFunction(functionTable functions){
	std::map<std::string,hAbstractFunction *>::iterator it;
	for(it=functions.begin();it!=functions.end();++it){
		data.insert(std::make_pair(it->first,it->second));
	}
}

std::vector<std::string> hFunction::getAllNames(void){
	functionTable::iterator it;
	std::vector<std::string> r;
	for(it=data.begin();it!=data.end();++it){
		r.push_back(it->first);
	}
	return r;
}

std::vector<std::string> hFunction::getToolBoxNames(void){
	std::list<hToolBox *>::iterator box;
	std::vector<std::string> names;
	hToolBox *temp;
	for(box=boxNames.begin();box!=boxNames.end();++box){
		temp=*box;
		names.push_back(temp->toolBoxName());
	}
	return names;
}

