/***************************************************************************
 *   Copyright (C) 2007 by Harry Serenis                                   *
 *   bobomastoras@users.sf.net                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef HFUNCTIONHEADERS_H
#define HFUNCTIONHEADERS_H

#include <map>
#include <vector>
#include <string>

#include "hVariable.h"
#include "hException.h"

class hFunction;
class hKernel;

typedef std::vector<hVariable> hStructure;

// abstract class describing a general function
class hAbstractFunction
{
protected:
	hFunction *tools;
public:
	hAbstractFunction(void){}
	void setExec(hFunction *exec){tools=exec;}

	// pure virtual function that is used to "execute" the function code
	// input and output is handled using a vector of hVariables
	virtual hStructure execute(hStructure &arg) = 0;
//	virtual hVariable execute(hVariable &arg)=0;

	// returns the name of the function
	virtual std::string getName(void)=0;

	// returns the number of input arguments
	virtual unsigned short getNumberOfArgs(void)=0;

	// returns the number of output arguments
	virtual unsigned short getNumberOfOut(void)=0;

	virtual std::string getHelp(void){return std::string("The developer of this toolBox is worthless and didn't take the time to implement a manual for it\n");}

};

class hToolBox{
protected:
	std::string name;
	std::map<std::string,hAbstractFunction *> functionNames;
public:
	virtual std::map<std::string,hAbstractFunction *> getNames(void)=0;

	std::vector<std::string> getNamesOnly(void){
		std::vector<std::string> toolNames;
		std::map<std::string,hAbstractFunction *>::iterator it;
		for(it=functionNames.begin();it!=functionNames.end();++it)
			toolNames.push_back(it->first);
		return toolNames;
	}

	virtual std::string toolBoxName(void)=0;
};

//typedef std::multimap<std::string,hAbstractFunction *> functionTable;
typedef std::map<std::string,hAbstractFunction *> functionTable;
#endif // hABSTRACTFUNCTION_H
