/***************************************************************************
 *   Copyright (C) 2007 by verminoz,Harry Serenis,manos,Christos Gkekas    *
 *   verminoz@users.sf.net,bobomastoras@users.sf.net,etsardou@users.sf.net,chggr@ee.auth.gr*
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "hKernel.h"

#define packageMath

class hLoopTemplate;

hKernel::hKernel(void){
	cout.precision(20);
	inStream=&std::cin;
	outStream=&std::cout;
// 	prompt=false;
	brackets=0;
	cmdExecutioner=hSystemCommand<hKernel>(this);
	initialize();
}

hKernel::hKernel(std::string home){
	cout.precision(20);
	inStream=&std::cin;
	outStream=&std::cout;
// 	prompt=false;
	brackets=0;
	cmdExecutioner=hSystemCommand<hKernel>(this);
	initialize(home);
}

hKernel::hKernel(istream& in,ostream& out){
	out.precision(20);
	inStream=&in;
	outStream=&out;
// 	prompt=false;
	brackets=0;
	cmdExecutioner=hSystemCommand<hKernel>(this);
	initialize();
}

void hKernel::initialize(std::string home){
	*outStream << "EngLab "VERSION"\n";
	*outStream << "Open source engineering/mathematics laboratory\n";
	*outStream << "Initializing EngLab...\n";
try{
	//initialize
	hGlobals::initialize(home);
	loadCmdHandlers();
	//initialize plot engine
	loadLoops();
	loadPackages();
	//runner.setKernel(this);
	queStack.push_back(hCommandBlock());

	parse("exec "+hGlobals::initFile+" -s",false,true);

	memory.insertNode("i",std::complex<float>(0,1),true);
	*outStream << "EngLab Ready!\n\nEnter command help for help\n\n>> ";
}catch(hException &e){
	e.printAll();
	*outStream << "EngLab Ready!\n\nEnter command help for help\n\n>> ";
}

/*
        //Initialize history.
        history.setNumSessions(5); //Set the number of sessions to 5.
        history.clearHistoryFiles();
        history.readHistoryData();
        history.createCurrentSession();
*/
}

void hKernel::initialize(){
	*outStream << "EngLab "VERSION"\n";
	*outStream << "Open source engineering/mathematics laboratory\n";
	*outStream << "Initializing EngLab...\n";
try{
	//initialize
	hGlobals::initialize();
	loadCmdHandlers();
	//initialize plot engine
	loadLoops();
	loadPackages();
	//runner.setKernel(this);
	queStack.push_back(hCommandBlock());
	//std::string autoexec=getenv("HOME")+autoe;

	parse("exec "+hGlobals::initFile+" -s",false,true);
	
	memory.insertNode("i",std::complex<float>(0,1),true);
	*outStream << "EngLab Ready!\n\nEnter command help for help\n\n>> ";
}catch(hException &e){
	*outStream << "EngLab Ready!\n\nEnter command help for help\n\n>> ";
	e.printAll();
}


        //Initialize history.
        history.setNumSessions(5); //Set the number of sessions to 5.
        history.clearHistoryFiles();
        history.readHistoryData();
        history.createCurrentSession();
	parse("exec "+hGlobals::initialcode+" -s",false,true);
}

void hKernel::terminate(void){
	*outStream << "Terminating EngLab...\n";
	clear();
	//delete plotEngine;
	//dlclose(h);
	*outStream << "\t\tI'll be bach!\n";

        //Write the history file.
        history.saveCurrentSession();
}

void hKernel::loadCmdHandlers(void){
	cmdExecutioner.addCommand("exit",hKernelCmdHandler::hTerminate);
	cmdExecutioner.addCommand("about",hKernelCmdHandler::hCredits);
	cmdExecutioner.addCommand("exec",hKernelCmdHandler::hFile);
	cmdExecutioner.addCommand("delete",hKernelCmdHandler::hClearVar);
	cmdExecutioner.addCommand("clear",hKernelCmdHandler::hClearGUI);
	cmdExecutioner.addCommand("lsmem",hKernelCmdHandler::hShowMem);
	cmdExecutioner.addCommand("history",hKernelCmdHandler::hShowHistory);
	cmdExecutioner.addCommand("!",hKernelCmdHandler::hPreviousCommand);
	cmdExecutioner.addCommand("lsfunc",hKernelCmdHandler::hShowFunctions);
	cmdExecutioner.addCommand("help",hKernelCmdHandler::hHelp);

}

void hKernel::loadLoops(void){
#define optional true
#define notOptional false
#define unique true
#define notUnique false

	//loopHandler=new hSystemLoop;
	hLoopTemplate loop;
	loop.addKeyword("for",notOptional,unique,3);
	loopHandler.addLoop(loop);
	loop.clear();
	loop.addKeyword("if",notOptional,unique,1);
	loop.addKeyword("else if",optional,notUnique,1);
	loop.addKeyword("else",optional,unique,1);
	loopHandler.addLoop(loop);
	loop.clear();
	loop.addKeyword("do",notOptional,unique,0);
	loop.addKeyword("while",notOptional,unique,1);
	loopHandler.addLoop(loop);
	loop.clear();
	loop.addKeyword("while",notOptional,unique,1);
	loopHandler.addLoop(loop);
	loop.clear();
#undef optional
#undef notOptional
#undef unique
#undef notUnique

}

void hKernel::loadPackages(void){
//	runner.setKernel(this);
	runner.loadToolBox(&package.math);
//	runner.loadToolBox(&package.stat);
//	runner.loadToolBox(&package.plotbox);
	runner.loadToolBox(&package.Conversions);
#ifdef UNIX
	void *h;
	hToolBox * (*p)(void);
	unsigned int size,i;
	size=hGlobals::solibnames.size();
	for(i=0;i<size;++i){
		*outStream<<"Loading library: "<<hGlobals::solibnames.at(i)<<" ...";
		dlerror();
		h=dlopen(hGlobals::solibnames.at(i).c_str(),RTLD_LAZY|RTLD_LOCAL);
		if(!h){
			*outStream<<"FAILED\n";
			*outStream<<"\tLoader error message: "<<dlerror()<<endl;
			*outStream<<"Attempting to load library using explicit path\n";
			*outStream<<"Loading shared object"<<hGlobals::solibpaths.at(i)<<" ...";
			dlerror();
			h=dlopen(hGlobals::solibpaths.at(i).c_str(),RTLD_LAZY|RTLD_LOCAL);
			if(!h){
				*outStream<<"FAILED\n";
				*outStream<<"\tError message: "<<dlerror()<<endl;
				*outStream<<"Toolbox will not be loaded!\n";
			}else{
				dlerror();
				p=(hToolBox * (*)(void))dlsym(h, "loader");
				const char* dlsym_error = dlerror();
				if (dlsym_error) {
					*outStream<<"FAILED\n";
					*outStream<<"\tCannot find loader function. Possibly incorrect englab toolbox\n"<< dlsym_error << '\n';
					*outStream<<"\tLoader error message: "<<dlerror()<<endl;
					*outStream<<"Toolbox will not be loaded!\n";
				}else{
					runner.loadToolBox(p());
					*outStream<<"DONE\n";
				}
			}
		}else{
			dlerror();
			p=(hToolBox * (*)(void))dlsym(h, "loader");
			const char* dlsym_error = dlerror();
			if (dlsym_error) {
				*outStream<<"FAILED\n";
				*outStream<<"\tCannot find loader function. Possibly incorrect englab toolbox\n"<< dlsym_error << '\n';
				*outStream<<"\tLoader error message: "<<dlerror()<<endl;
				*outStream<<"Toolbox will not be loaded!\n";
			}else{
				runner.loadToolBox(p());
				*outStream<<"DONE\n";
			}
		}
	}
#endif

}


void hKernel::console(void){
	unsigned char ch;
	int i;
//	FILE *out;
	std::string input;
// 	*outStream<<">> ";
//	out=fopen("debug.opg","w");
//	fclose(out);
	do{
//		out=fopen("debug.opg","a");
		for(i=0;i<brackets;i++)
			*outStream<<'\t';
		ch=inStream->get();
		while(ch!='\n'){
//			fputc(ch,out);
			input+=ch;
			ch=inStream->get();
		}
//		fputc(13,out);
//		fclose(out);
		if(!isExprNull(input)){
                	history.addCommand(input);
	                parse(input,false,false);
		}
		else *outStream<<">> ";
                input.clear();
	}while(1);
}

int hKernel::execute(hAbstractCommand cmd,bool silent){
	hAbstractCommand simple;
	int ret=0;
	while(!cmd.empty()){
	simple=cmd.front();
	cmd.pop();
	bool gotExecuted=false;
	try{
//		cmdQue=hGlobals::trim(cmdQue);
		prompt=true;
		if(loopHandler.waitForCmd()){
			loopHandler.addCmd(simple);
			gotExecuted=true;
			if(loopHandler.complete()){
				loopHandler.execute(this);
				loopHandler.reset();
				if(!silent)
				*outStream<<">> ";
			}
		}
		string input=simple.getCommand();
		if(!gotExecuted){
			loopHandler.loadKey(input);
			if(loopHandler.complete()){
				loopHandler.execute(this);
				if(!silent)
					*outStream<<">> "<<input<<std::endl;
			}else if(loopHandler.valid()){
				gotExecuted=true;
			}else{
				loopHandler.reset();
			}
		}
	
		if(!gotExecuted){
			cmdExecutioner.setCommand(input);
			if(cmdExecutioner.isValid()){
				ret=cmdExecutioner.execute();
				if(!silent & prompt)
					*outStream<<">> ";
				gotExecuted=true;
			}
		}
		if(!gotExecuted){
			hVariable r=cmdParser.parseCommand(input,&memory,&runner);
			if(cmdParser.lastName!=""){
				hNodeTreeVariable *b;
				b=memory.returnNode(cmdParser.lastName);
				if(cmdParser.respond() && silent==false){
					*outStream<<"\t"<<cmdParser.lastName<<" = "<<(b->returnVar())<<"\n";

				}
			}
			if(!silent)
				*outStream<<">> ";
		}
/*		if(!gotExecuted){
			*outStream<<"\tError! Unknown Command: "<<input<<"\n";
			if(!silent)
				*outStream<<">> ";
		}*/
	}catch(hException &e){
		e.printAll();
		e.clear();
		*outStream<<">> ";
	}
	}
	return ret;
}

/*int hKernel::execute(bool silent){
	bool gotExecuted,isBracket;
	int ret=0;
	//if(i=(a==0){
try{
	if(brackets<0){
        brackets=0;
		throw hException("hKernel::parse","Unexpected bracket '}'",hBracket);
    }
	string input;
	while(queStack.size()){
	//while(cmdQue.size()){
		//input=*cmdQue.begin();
		gotExecuted=isBracket=false;
		prompt=true;
		if(!input.compare("{") || !input.compare("}"))
			isBracket=true;
		if(!gotExecuted){
			if(loopHandler.waitForCmd()){
				//loopHandler.addCmd(cmdQue);
				loopHandler.addCmd(queStack.back());
				queStack.back().pop();
				//cmdQue.clear();
				gotExecuted=true;
				if(loopHandler.complete()){
					loopHandler.execute(this);
					loopHandler.reset();
					if(!silent)
						*outStream<<">> ";
				}
			}
		}
		input=queStack.back().getCommand();
		queStack.back().pop();
		if(!gotExecuted){
				loopHandler.loadKey(input);
				if(loopHandler.complete()){
					loopHandler.execute(this);
						if(!silent)
							*outStream<<">> "<<input<<std::endl;
				}else if(loopHandler.valid()){
					gotExecuted=true;
				}else{
					loopHandler.reset();
				}
		}

		if(!gotExecuted){
			cmdExecutioner.setCommand(input);
			if(cmdExecutioner.isValid()){
				//prompt=false;
				ret=cmdExecutioner.execute();
				//if(!prompt){
					if(!silent & prompt)
						*outStream<<">> ";
					//prompt=true;
				//}
				gotExecuted=true;
			}
		}
		if(!gotExecuted){
			hVariable r=cmdParser.parseCommand(input,&memory,&runner);
			gotExecuted=true;
			if(gotExecuted){
				if(cmdParser.lastName!=""){
				hNodeTreeVariable *b;
				b=memory.returnNode(cmdParser.lastName);
				if(cmdParser.respond() && silent==false){
					//if((r=memory.returnNode("ans"))!=NULL){
						*outStream<<"\t"<<cmdParser.lastName<<" = "<<(b->returnVar())<<"\n";
					//}
				}
				}
				if(!silent)
					*outStream<<">> ";
			}
		}
		if(!gotExecuted){
			*outStream<<"\tError! Unknown Command: "<<input<<"\n";
// 			if(!prompt){
				if(!silent)
					*outStream<<">> ";
// 				prompt=true;
// 			}
		}
	}
	queStack.clear();
	queStack.push_back(hCommandBlock());
}catch(hException &e){
	e.printAll();
	e.clear();
	*outStream<<">> ";
}
	return ret;
}*/

void hKernel::loadToolBox(std::string name){
	unsigned int i,s;
	std::string names;
	//names=hGlobals::solibnames;
	s=names.size();
/*	for(i=0;i<size;++i){
		*outStream<<"Loading shared object"<<hGlobals::solibnames.at(i)<<" ...";
		dlerror();
		h=dlopen(hGlobals::solibnames.at(i).c_str(),RTLD_NOW);
		if(!h){
			*outStream<<"FAILED\n";
			*outStream<<"\tLoader error message: "<<dlerror()<<endl;
			*outStream<<"Attempting to load library using explicit path\n";
			*outStream<<"Loading shared object"<<hGlobals::solibpaths.at(i)<<" ...";
			dlerror();
			h=dlopen(hGlobals::solibpaths.at(i).c_str(),RTLD_NOW);
			if(!h){
				*outStream<<"FAILED\n";
				*outStream<<"\tError message: "<<dlerror()<<endl;
				*outStream<<"Toolbox will not be loaded!\n";
			}else{
				dlerror();
				p=(hToolBox * (*)(void))dlsym(h, "loader");
				const char* dlsym_error = dlerror();
				if (dlsym_error) {
					*outStream<<"FAILED\n";
					*outStream<<"\tCannot find loader function. Possibly incorrect englab toolbox\n"<< dlsym_error << '\n';
					*outStream<<"\tLoader error message: "<<dlerror()<<endl;
					*outStream<<"Toolbox will not be loaded!\n";
				}else{
					runner.loadToolBox(p());
					*outStream<<"DONE\n";
				}
			}
		}else{
			dlerror();
			p=(hToolBox * (*)(void))dlsym(h, "loader");
			const char* dlsym_error = dlerror();
			if (dlsym_error) {
				*outStream<<"FAILED\n";
				*outStream<<"\tCannot find loader function. Possibly incorrect englab toolbox\n"<< dlsym_error << '\n';
				*outStream<<"\tLoader error message: "<<dlerror()<<endl;
				*outStream<<"Toolbox will not be loaded!\n";
			}else{
				runner.loadToolBox(p());
				*outStream<<"DONE\n";
			}
		}
	}*/
			
}

void hKernel::execFile(std::string name,bool silent){
	unsigned char ch;
	int i;
	std::string input;
	std::ifstream infile;
	infile.open(name.c_str(),std::ifstream::in);
	if(infile.fail()){
		throw hException("hKernel::execFile","Cannot open file: "+name,hError);
	}
	if(!silent)
		*outStream<<">> ";
	while(infile.good()){
		ch=infile.get();
		while(ch!='\n' && infile.good()){
			input+=ch;
			ch=infile.get();
		}
		if(input.size())
			parse(input,!silent,silent);
		input.clear();
	}
	infile.close();
	prompt=silent;
}


void hKernel::clear(void){
	memory.deleteTree();
}

void hKernel::clearVars(std::vector<std::string> names){
	unsigned int i;
	if(names.size()>0){
		for(i=0;i<names.size();i++)
			memory.deleteNode(names[i],false);
	}else{
		memory.deleteVars();
	}
}

void hKernel::showVars(void){
	hNodeTreeVariable** data;
	int varnum =  memory.returnIndext();
	data = new hNodeTreeVariable*[varnum];
	data = memory.returnAllData(data);
	unsigned int dims;
	std::vector<unsigned int> siz;
	*outStream<<"Name" << "\t\t"<< "Size"<<"\t\t"<<"Data Type"<<std::endl;
	*outStream<<"____" << "\t\t"<< "____"<<"\t\t"<<"_________"<<std::endl;
	for (int i = 0; i < varnum; i++) {
		if(! data[i]->getConstant()){
			dims = data[i]->returnVar().getDim();
			siz = data[i]->returnVar().getSize();
			*outStream << std::endl << data[i]->returnName() << "\t\t";
			for (unsigned int j=0; j<dims; j++) {
				*outStream << siz[j];
				if (j!=dims-1) {
					*outStream << "x";
				}
			}
//			*outStream<<"\t\t"<<int(data[i]->returnVar().getType());
			*outStream<<"\t\t"<<hVariable::literalType(data[i]->returnVar().getType());

		}
	}
	*outStream<<std::endl;
	delete []data;
}

void hKernel::showFunctions(void){
	std::vector<std::string> names;
	unsigned int i;
	*outStream<<"Available Functions\n";
	names=runner.getAllNames();
	for(i=0;i<names.size();++i){
		*outStream<<names[i]<<endl;
	}
}



hVariableTree hKernel::returnTree(void){
	return memory;
}

void hKernel::setOutStream(std::ostream &out){
	outStream=&out;
}

int hKernel::parse(hAbstractCommand in,bool echo,bool silent){
//	queStack.back().push(in);
	execute(in,silent);
}

int hKernel::parse(std::string in,bool echo,bool silent){
	unsigned int i;
	std::string cmd;
	if(echo)
		*outStream<<in<<"\n";
	if(!isExprNull(in)){	
		for(i=0;i<in.size();i++){
			if(in[i]=='{'){
				if(cmd.size()){
					queStack.back().push(hAbstractCommand(cmd));
					cmd.clear();
				}
				queStack.push_back(hCommandBlock());
			}else if(in[i]=='}'){
				if(cmd.size()){
					queStack.back().push(hAbstractCommand(cmd));
					cmd.clear();
				}
				hCommandBlock temp=queStack.back();
				queStack.pop_back();
				queStack.back().push(temp);
			}else{
				cmd+=in[i];
			}
		}
		if(cmd.size())
			queStack.back().push(cmd);
		if(queStack.size()==1){
			int rets=0;
//			for(i=0;i<queStack.back().size();i++){
				hAbstractCommandBase *t;
				t=new hCommandBlock(queStack.back());
				queStack.clear();
				queStack.push_back(hCommandBlock());
				return execute(t,silent);
//			}
		}else return 0;
	}
}



void hKernel::credits(void){
	cout << "EngLab Credits:\n\n";
	cout << "Athanasios Vogiannou\taka\tPoumpa\n";
	cout << "Charalambos Serenis\taka\tBOBoMASTORAS\n";
	cout << "Efstratios Gavves\taka\tAurelius\n";
	cout << "Emmanouil Tsardoulias\taka\tKamenos\n";
	cout << "Nikolaos Parastatidis\taka\tKenny\n";
}

std::vector<std::string> hKernel::getFunctionNames(void){
	return runner.getAllNames();
}

std::vector<std::string> hKernel::getConsoleCommands(void){
	return cmdExecutioner.getCommandNames();
}

std::vector<std::string> hKernel::getToolBoxNames(void){
	return runner.getToolBoxNames();
}

std::vector<std::string> hKernel::getToolBoxFunctions(std::string name){
	return runner.getToolBoxFunctions(name);
}


int hKernelCmdHandler::hTerminate(hKernel *obj,vector<string> argv){
	obj->terminate();
	exit(0);
}

int hKernelCmdHandler::hCredits(hKernel *obj,vector<string> argv){
	obj->credits();

	return 0;
}

int hKernelCmdHandler::hFile(hKernel *obj,std::vector<std::string> argv){
	bool s=true;
	if(argv.size()==1)
		throw hException("hKernelCmdHandler::hFile","No input argument",hError);
	if(argv.size()>2 && !argv[2].compare("-v"))
		s=false;
	obj->execFile(argv[1],s);

	return 0;
}

int hKernelCmdHandler::hClearVar(hKernel *obj,std::vector<std::string> argv){
	std::string temp;
	temp=argv[0];
	if(argv.size()>1){
		argv[0]=argv[argv.size()-1];
		argv[argv.size()-1]=temp;
	}
	temp.clear();
	argv.pop_back();
	obj->clearVars(argv);

	return 0;
}

int hKernelCmdHandler::hClearGUI(hKernel *obj,std::vector<std::string> argv){
	if(argv.size()==1){
		return hKernelCmdHandler::hClearAll;
	}else if(argv.size()==2){
		if(!argv[1].compare("history")){
			obj->history.deleteHistory();
			obj->history.clearHistoryFiles();
			return hKernelCmdHandler::hClearHistory;
		}else if(!argv[1].compare("output"))
			return hKernelCmdHandler::hClearOutput;
		else
			throw hException("hKernelCmdHandler::hClearGUI","Invalid Argument",hBadArg);
	}else

	return 0;
}

int hKernelCmdHandler::hShowMem(hKernel *obj,std::vector<std::string> argv){
	obj->showVars();
	return 0;
}

int hKernelCmdHandler::hHelp(hKernel *obj,std::vector<std::string> argv){
	std::ostream *out;
	out=obj->getOStream();
	if(argv.size()==1){
		std::vector<std::string> names;
		*out<<"Console Commands:\n";
		names=obj->getConsoleCommands();
		unsigned int size=names.size();
		for(unsigned int i=0;i<size;++i){
			*out<<'\t'<<names[i]<<std::endl;
		}
		*out<<"\nToolBoxes:\n";
		names=obj->getToolBoxNames();
		size=names.size();
		for(unsigned int i=0;i<size;++i){
			*out<<'\t'<<names[i]<<std::endl;
		}
	}else if(argv.size()==2){
		std::vector<std::string> names;
/*		names=obj->getConsoleCommands();
		unsigned int size=names.size();
		for(unsigned int i=0;i<size;++i){
			*out<<'\t'<<names[i]<<std::endl;
		}
		*out<<"\nToolBoxes:\n";*/
		names=obj->getToolBoxNames();
		unsigned int size=names.size();
		for(unsigned int i=0;i<size;++i){
			if(!names[i].compare(argv[1])){
				*out<<"Functions of toolbox "<<names[i]<<":\n";
				names=obj->getToolBoxFunctions(names[i]);
				size=names.size();
				for(unsigned int i=0;i<size;++i){
					*out<<'\t'<<names[i]<<std::endl;
				}
				*out<<"Type help \"toolbox\" \"function\" for more help\n";
				break;
			}
		}
	}else if(argv.size()==3){
		std::string temp;
		temp=argv[1]+' '+argv[2];
		*out<<obj->getHelp(temp);
	}
}


/**
 *  @brief Prints the command history.
 *
 *  This is a simple method used to print the command history. It is executed only when the user
 *  issues the command "history".
 */
int hKernelCmdHandler::hShowHistory(hKernel *obj,std::vector<std::string> argv)
{
  obj->history.printHistoryData();
  return 0;
}

/**
 *  @brief Runs one or more previous commands.
 *
 *  This method is used to run one or more previous commands found inside the command history. It is
 *  called by issuing an exclamation mark (!) at the beginning and then the serial number of the
 *  command/s the user wants to be executed. The method searches the command history to find the
 *  specified commands and then runs them one by one.
 */
int hKernelCmdHandler::hPreviousCommand(hKernel *obj,std::vector<std::string> argv)
{
  //A string variable is declared which is going to hold the previous command.
  string previousCommand;

  //Perform a sanity check. If the first arguement is not an exclamation mark, return with an error code.
  if (argv.at(0) != "!") return 1;
  if (argv.size()==1){
	obj->history.printHistoryData();
  	return 0;
  }
  //For every command serial number given, search the history, find the requested command and execute it.
  for (int i=1; i<argv.size(); i++)
  {
    previousCommand = obj->history.getHistoryCommand(atoi(argv.at(i).c_str()));
    obj->history.addCommand(previousCommand);
    obj->parse(previousCommand,true,true);
  }
  return 0;
}

int hKernelCmdHandler::hShowFunctions(hKernel *obj,std::vector<std::string> argv)
{
	if(argv.size()!=1){
		throw hException("hKernelCmdHandler::hClearGUI","Invalid Argument",hBadArg);
	}
	obj->showFunctions();
  return 0;
}

int hKernelCmdHandler::hLoadToolBox(hKernel *obj,std::vector<std::string> argv){
	if(argv.size()==1){
		throw hException("hKernelCmdHandler::hLoadToolBox","You must specify a toolbox",hBadArg);
	}
	unsigned int s,i;
	s=argv.size();
	for(i=1;i<s;++i){
		obj->loadToolBox(argv[i]);
	}
}

bool hKernel::isExprNull(std::string input)
{
	int i=0;
	while(i<input.size())
	{
		if(input[i]!=' ' && input[i]!=';')
			return false;
		i++;
	}
	return true;
}

std::string hKernel::getHelp(std::string argv){
	using namespace std;
	return runner.getHelp(hGlobals::parseSpace(argv));
}

