/***************************************************************************
 *   Copyright (C) 2007 by verminoz,Harry Serenis,manos,Christos Gkekas    *
 *   verminoz@users.sf.net,bobomastoras@users.sf.net,etsardou@users.sf.net,chggr@ee.auth.gr*
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef HKERNEL_H
#define HKERNEL_H

#include <iostream>
#include <fstream>
#include <string>
#include <list>
#include <sys/types.h>
#include <time.h>
#include "hGlobals.h"
#include "hVariableTree.h"
#include "hSystemCommand.h"
#include "hCommandParser.h"
#include "hLoopParser.h"
#include "hException.h"
#include "hCommandHistory.h"
#include "hFunction.h"
#include "config.h"
#include "hCommandBlock.h"
//#include "hPlotEngine.h"
//#include "hPlotBox.h"
#ifdef UNIX
#include <dlfcn.h>
#endif
//#include "hParametricExecution.h"



struct kernelPackages{
	hMath math;
	hConversions Conversions;
};

class hKernel;

namespace hKernelCmdHandler{
	enum{
		hSuccess=0,
		hClearHistory,
		hClearOutput,
		hClearAll
	};

	int hTerminate(hKernel *obj,std::vector<std::string> argv);
	int hCredits(hKernel *obj,std::vector<std::string> argv);
	int hFile(hKernel *obj,std::vector<std::string> argv);
	int hClearVar(hKernel *obj,std::vector<std::string> argv);
	int hClearGUI(hKernel *obj,std::vector<std::string> argv);
	int hShowMem(hKernel *obj,std::vector<std::string> argv);
	int hShowHistory(hKernel *obj,std::vector<std::string> argv);
	int hPreviousCommand(hKernel *obj,std::vector<std::string> argv);
	int hShowFunctions(hKernel *obj,std::vector<std::string> argv);
	int hHelp(hKernel *obj,std::vector<std::string> argv);
	int hLoadToolBox(hKernel *obj,std::vector<std::string> argv);

}


class hKernel{
	kernelPackages package;
	hFunction runner;
	void initialize(void);
	void initialize(std::string home);
	hSystemCommand<hKernel> cmdExecutioner;
	hSystemLoop loopHandler;
	hCommandParser cmdParser;
	//hCodeBlock cmdQue;
	std::vector<hCommandBlock> queStack;
	int brackets;
	bool prompt;
	void clearVars(std::vector<std::string> args);
	void loadCmdHandlers(void);
	void loadLoops(void);
	void loadPackages(void);
//	int execute(bool silent);
	int execute(hAbstractCommand cmd,bool silent);
	istream *inStream;
	ostream *outStream;
	friend class hLoopsExecution;
	friend int hKernelCmdHandler::hClearVar(hKernel *obj,std::vector<std::string> argv);
	bool isExprNull(std::string input);
public:
        //HISTORY
        hCommandHistory history;
	hVariableTree memory;
	hKernel(void);
	hKernel(std::istream &in,std::ostream &out);
	hKernel(std::string home);
	void setOutStream(std::ostream &out);
// 	void internalParse(std::string in);
// 	void fileParse(std::string,bool silent);
	int parse(std::string in,bool echo,bool silent);
	int parse(hAbstractCommand in,bool echo,bool silent);
	void execFile(std::string name,bool silent);
	void clear(void);
	hVariableTree returnTree(void);
	void terminate(void);
	void console(void);
	void credits(void);
	void showVars(void);
	void showFunctions(void);
	ostream * getOStream(void){return outStream;}
	std::vector<std::string> getFunctionNames(void);
	std::vector<std::string> getConsoleCommands(void);
	std::vector<std::string> getToolBoxNames(void);
	std::vector<std::string> getToolBoxFunctions(std::string name);
	std::string getHelp(std::string argv);
	hFunction* getRunner(void){return &runner;}
	void loadToolBox(std::string name);
};


#endif
