/***************************************************************************
 *   Copyright (C) 2007 by Harry Serenis                                   *
 *   bobomastoras@users.sf.net                                             * 
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "hKernel.h"


hLoopTemplate::hLoopTemplate(void){
}

void hLoopTemplate::addKeyword(std::string param,bool isOptional,bool isUnique,unsigned int nOfFields){
	keyword.push_back(param);
	optional.push_back(isOptional);
	unique.push_back(isUnique);
	condition.push_back(nOfFields);
}

void hLoopTemplate::clear(void){
	keyword.clear();
	optional.clear();
	unique.clear();
	condition.clear();
}

void hSystemLoop::addLoop(hLoopTemplate newLoop){
	loop.push_back(newLoop);
}

hSystemLoop::hSystemLoop(void){
	cmdWait=false;
	isValid=false;
	isComplete=false;
	loopId=-1;
	keywordId=0;
}

void hSystemLoop::addCmd(hAbstractCommand input){
	exec=false;
	if(cmdWait){
		exec=true;
		cmdWait=false;
		parsedLoop.setExecution(loop.at(loopId).keyword.at(keywordId),params,input);
		params.clear();
		if(loop.at(loopId).unique.at(keywordId)){
			keywordId++;
			if(keywordId==loop.at(loopId).keyword.size()){
				isComplete=true;
			}
		}
	}
}

/*void hSystemLoop::addCmd(hCodeBlock input){
	std::vector<std::string> cmd;	
	hCodeBlock::iterator it;
	exec=false;
	if(cmdWait){
		if(!input.begin()->compare("{") && !input.rbegin()->compare("}")){
			input.pop_front();
			input.pop_back();
		}
		for(it=input.begin();it!=input.end();it++)
			cmd.push_back(*it);
		exec=true;
		cmdWait=false;
		parsedLoop.setExecution(loop.at(loopId).keyword.at(keywordId),params,cmd);
		cmd.clear();
		params.clear();
		if(loop.at(loopId).unique.at(keywordId)){
			keywordId++;
			if(keywordId==loop.at(loopId).keyword.size()){
				isComplete=true;
			}
		}
	}
}*/

void hSystemLoop::loadKey(std::string key){
	int i;
	if(!cmdWait){
		std::string arg;
		i=key.find('(',0);
		if(i!=-1){
			arg=key.substr(0,i);
			arg=hGlobals::trim(arg);
			params=key.substr(i,key.size()-i);
			params=hGlobals::trim(params);
		}else{
			arg=key.substr(0,key.size());
		}
		i=loopId ? loopId!=-1 : 0;
		while(i<loop.size() && !cmdWait && !isComplete){
			if(!loop.at(i).keyword.at(keywordId).compare(arg)){
				exec=true;
				cmdWait=true;
				isValid=true;
				isComplete=false;
				loopId=i;
			}else if(isValid && loop.at(i).optional.at(keywordId)){
				keywordId++;
				if(keywordId==loop.at(i).keyword.size()){
					isComplete=true;
				}
			}
			if(!isValid)
				i++;
		}
	}
}

//class hLoopExecution;

void hSystemLoop::execute(hKernel *kernel){
	hLoopsExecution loopExecutioner;
	loopExecutioner.setKernel(kernel);
	loopExecutioner.setParametricExecution(parsedLoop);
	reset();
	loopExecutioner.execute();
/*
	using namespace std;
	unsigned int i,j;
	vector<string> keywords;
	vector<vector<string> > params;
	vector<vector<string> > cmds;
	keywords=parsedLoop.getKeywords();
	params=parsedLoop.getParams();
	cmds=parsedLoop.getCmds();

	for(i=0;i<keywords.size();i++){
		cout<<keywords[i]<<" with parameters";
		for(j=0;j<params[i].size();j++)
			cout<<" : "<<params[i][j];
		cout<<endl<<cmds[i].size()<<" Commands:\n";
		for(j=0;j<cmds[i].size();j++)
			cout<<cmds[i][j]<<endl;
		cout<<endl;
	}
 	reset();*/
}

bool hSystemLoop::valid(void){
	return isValid;
}


bool hSystemLoop::complete(void){
	return isComplete;
}

bool hSystemLoop::waitForCmd(void){
	return cmdWait;
}

void hSystemLoop::reset(void){
// 	for(i=0;i<loop.size();i++)
// 		loop[i].clear();
// 	loop.clear();
	params.clear();
	parsedLoop.clear();
	isValid=false;
	isComplete=false;
	cmdWait=false;
	loopId=-1;
	keywordId=0;
}

bool hSystemLoop::gotExecuted(void){
	return exec;
}



