/***************************************************************************
 *   Copyright (C) 2007 by Harry Serenis                                   *
 *   bobomastoras@users.sf.net                                             * 
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef HLOOPPARSER_H
#define HLOOPPARSER_H

#include <iostream>
#include <vector>
#include <string>
#include "hGlobals.h"
#include "hException.h"
#include "hParametricExecution.h"
#include "hLoops.h"
#include "hKernel.h"

class hKernel;

class hLoopTemplate{
public:
	std::vector<std::string> keyword;
	std::vector<bool> optional,unique;
	std::vector<unsigned int> condition;
	hLoopTemplate(void);
	void addKeyword(std::string param,bool isOptional,bool isUnique,unsigned int nOfFields);
	void clear(void);
};

//typedef std::deque<std::string> hCodeBlock;
class hSystemLoop{
	std::vector<hLoopTemplate> loop;
	std::string params;
	hParametricExecution parsedLoop;
	bool isValid;
	bool isComplete;
	bool cmdWait;
	bool exec;
	unsigned int loopId;
	unsigned int keywordId;
public:
	void reset(void);
	hSystemLoop(void);
	void addLoop(hLoopTemplate newLoop);
	void loadKey(std::string key);
	//void addCmd(hCodeBlock input);
	void addCmd(hAbstractCommand input);
	void execute(hKernel *kernel);
	void clear(void);
	bool valid(void);
	bool complete(void);
	bool waitForCmd(void);
	bool gotExecuted(void);
};	

typedef std::deque<std::string> hCodeBlock;

namespace hLoopParser{
enum{
	start=0,
	ifCondition,
	ifCode,
	elseCode,
	simpleCondition,
	simpleCode,
	doCode,
	doCondition,
	end
};

enum{
	nothing=0,
	hIf,
	hWhile,
	hFor,
	hDo
};

enum{
	idle=0,
	cmdWait
};

class hState{
	unsigned char FSMState,clause,state;
	std::vector<std::string> condition;
	std::vector<hCodeBlock> code;
};


}

class hLoopFSM{
	unsigned int state;
	unsigned int clause;
	unsigned int commState;
	std::vector<std::string> condition;
	std::vector<hCodeBlock> code;
public:
	hLoopFSM(void);
	unsigned char getState(void);
	void addCmd(hCodeBlock cmd);
	void execute(hKernel *kernel);
};

#endif
