/***************************************************************************
 *   Copyright (C) 2007 by manos,Harry Serenis                             *
 *   etsardou@users.sf.net,bobomastoras@users.sf.net                       * 
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "hMath.h"


hMath::hMath(void){
	name="math";
	functionNames["sin"]=&mathSin;
	functionNames["sinh"]=&mathSinh;
	functionNames["asin"]=&mathAsin;

	functionNames["cos"]=&mathCos;
	functionNames["cosh"]=&mathCosh;
	functionNames["acos"]=&mathAcos;

	functionNames["tan"]=&mathTan;
	functionNames["tanh"]=&mathTanh;
	functionNames["atan"]=&mathAtan;
	
	functionNames["abs"]=&mathAbs;

	functionNames["ceil"]=&mathCeil;
	//functionNames["exp"]=&mathExp;
	functionNames["floor"]=&mathFloor;
	//functionNames["fmod"]=&mathFmod;
	//functionNames["ldexp"]=&mathLdexp;
	functionNames["log"]=&mathLog;
	functionNames["log10"]=&mathLog10;
	//functionNames["pow"]=&mathPow;
	functionNames["sqrt"]=&mathSqrt;
}

std::map<std::string,hAbstractFunction *> hMath::getNames(void){
	return functionNames;
}

std::string hMath::toolBoxName(void){
	return name;
}


hStructure hSin::execute(hStructure &arg){
	hStructure ret;
	if(arg.size()==0)
		throw hException("hSin::execute","Cannot calculate function with zero input arguments",hError);
	ret.reserve(arg.size());
	ret.push_back(arg[0].traverse((double (*)(double))std::sin));
	return ret;
}
std::string hSin::getHelp(void){
	return "Syntax: sin(in)\n\nDescription: Function sin returns the trigonometrical sine of it's input arguments.\n\nInput parameters: 'in' can be a variable of any dimensions and any type, except for complex.\n\nExceptions: hException::hError if input argument's number is 0.\n\nReturns: a variable with the same dimensions as 'in', and of type double.\n\n";
}

hStructure hSinh::execute(hStructure &arg){
	hStructure ret;
	if(arg.size()==0)
		throw hException("hSinh::execute","Cannot calculate function with zero input arguments",hError);
	ret.reserve(arg.size());
	ret.push_back(arg[0].traverse((double (*)(double))std::sinh));
	return ret;
}
std::string hSinh::getHelp(void){
	return "Syntax: sinh(in)\n\nDescription: 	Function sinh returns the hyperbolic sine of it's input arguments. Hyperbolic sine's analytical form is: sinh(x)=(exp(x)-exp(-x))/2\n\nInput parameters: 'in' can be a variable of any dimensions and any type, except for complex.\n\nExceptions: hException::hError if input argument's number is 0.\n\nReturns: a variable with the same dimensions as 'in', and of type double.\n\n";
}

hStructure hCos::execute(hStructure &arg){
	hStructure ret;
	if(arg.size()==0)
		throw hException("hCos::execute","Cannot calculate function with zero input arguments",hError);
	ret.reserve(arg.size());
	ret.push_back(arg[0].traverse((double (*)(double))std::cos));
	return ret;
}
std::string hCos::getHelp(void){
	return "Syntax: cos(in)\n\nDescription: Function cos returns the trigonometrical cosine of it's input arguments.\n\nInput parameters: 'in' can be a variable of any dimensions and any type, except for complex.\n\nExceptions: hException::hError if input argument's number is 0.\n\nReturns: a variable with the same dimensions as 'in', and of type double.\n\n";
}

hStructure hCosh::execute(hStructure &arg){
	hStructure ret;
	if(arg.size()==0)
		throw hException("hCosh::execute","Cannot calculate function with zero input arguments",hError);
	ret.reserve(arg.size());
	ret.push_back(arg[0].traverse((double (*)(double))std::cosh));
	return ret;
}
std::string hCosh::getHelp(void){
	return "Syntax: cosh(in)\n\nDescription: Function cosh returns the hyperbolic cosine of it's input arguments. Hyperbolic cosine's analytical form is: sinh(x)=(exp(x)+exp(-x))/2\n\nInput parameters: 'in' can be a variable of any dimensions and any type, except for complex.\n\nExceptions: hException::hError if input argument's number is 0.\n\nReturns: a variable with the same dimensions as 'in', and of type double.\n\n";
}

hStructure hTan::execute(hStructure &arg){
	hStructure ret;
	if(arg.size()==0)
		throw hException("hTan::execute","Cannot calculate function with zero input arguments",hError);
	ret.reserve(arg.size());
	ret.push_back(arg[0].traverse((double (*)(double))std::tan));
	return ret;
}
std::string hTan::getHelp(void){
	return "Syntax: tan(in)\n\nDescription: Function tan returns the trigonometrical tangent of it's input arguments.\n\nInput parameters: 'in' can be a variable of any dimensions and any type, except for complex.\n\nExceptions: hException::hError if input argument's number is 0.\n\nReturns: a variable with the same dimensions as 'i', and of type double.\n\n";
}

hStructure hTanh::execute(hStructure &arg){
	hStructure ret;
	if(arg.size()==0)
		throw hException("hTanh::execute","Cannot calculate function with zero input arguments",hError);
	ret.reserve(arg.size());
	ret.push_back(arg[0].traverse((double (*)(double))std::tanh));
	return ret;
}
std::string hTanh::getHelp(void){
	return "Syntax: tanh(in)\n\nDescription: Function tanh returns the hyperbolic tangent of it's input arguments. Hyperbolic tangent's analytical form is: tanh(x)=sinh(x)/cosh(x)\n\nInput parameters: 'in' can be a variable of any dimensions and any type, except for complex.\n\nExceptions: hException::hError if input argument's number is 0.\n\nReturns: a variable with the same dimensions as 'in', and of type double.\n\n";
}

hStructure hAbs::execute(hStructure &arg){
	hStructure ret;
	if(arg.size()==0)
		throw hException("hAbs::execute","Cannot calculate function with zero input arguments",hError);
	ret.reserve(arg.size());
	ret.push_back(arg[0].traverse((double (*)(double))std::abs));
	return ret;
}
std::string hAbs::getHelp(void){
	return "Syntax: abs(in)\n\nDescription: In mathematics, the absolute value (or modulus which is Latin for a small measure) of a real number is its numerical value without regard to its sign. So, for example, 3 is the absolute value of both 3 and -3. In computer programming, the mathematical function used to perform this calculation is usually given the name abs().\n\nInput parameters: 'in' can be a variable of any dimensions and any type, except for complex.\n\nExceptions: hException::hError if input argument's number is 0.\n\nReturns: a variable with the same dimensions as 'in', and of type double.\n\n ";
}

hStructure hAcos::execute(hStructure &arg){
	hStructure ret;
	if(arg.size()==0)
		throw hException("hAcos::execute","Cannot calculate function with zero input arguments",hError);
	ret.reserve(arg.size());
	ret.push_back(arg[0].traverse((double (*)(double))std::acos));
	return ret;
}
std::string hAcos::getHelp(void){
	return "Syntax: acos(in)\n\nDescription: In mathematics, the inverse trigonometric functions are the inverse functions of the trigonometric functions. The function acos is the inverse function of cos. The function acos(x) returns the arc y for which cos(y)=x.\n\nInput parameters: 'i' can be a variable of any dimensions and any type, except for complex.\n\nExceptions: hException::hError if input argument's number is 0.\n\nReturns: a variable with the same dimensions as 'in', and of type double.\n\n";
}

hStructure hAsin::execute(hStructure &arg){
	hStructure ret;
	if(arg.size()==0)
		throw hException("hAsin::execute","Cannot calculate function with zero input arguments",hError);
	ret.reserve(arg.size());
	ret.push_back(arg[0].traverse((double (*)(double))std::asin));
	return ret;
}
std::string hAsin::getHelp(void){
	return "Syntax: asin(in)\n\nDescription: In mathematics, the inverse trigonometric functions are the inverse functions of the trigonometric functions. The function acos is the inverse function of cos. The function asin(x) returns the arc y for which sin(y)=x.\n\nInput parameters: 'in' can be a variable of any dimensions and any type, except for complex.\n\nExceptions: hException::hError if input argument's number is 0.\n\nReturns: a variable with the same dimensions as 'in', and of type double.\n\n";
}

hStructure hAtan::execute(hStructure &arg){
	hStructure ret;
	if(arg.size()==0)
		throw hException("hAtan::execute","Cannot calculate function with zero input arguments",hError);
	ret.reserve(arg.size());
	ret.push_back(arg[0].traverse((double (*)(double))std::atan));
	return ret;
}
std::string hAtan::getHelp(void){
	return "Syntax: atan(in)\n\nDescription: In mathematics, the inverse trigonometric functions are the inverse functions of the trigonometric functions. The function acos is the inverse function of cos. The function atan(x) returns the arc y for which tan(y)=x.\n\nInput parameters: 'in' can be a variable of any dimensions and any type, except for complex.\n\nExceptions: hException::hError if input argument's number is 0.\n\nReturns: a variable with the same dimensions as 'in', and of type double.\n\n";
}

hStructure hCeil::execute(hStructure &arg){
	hStructure ret;
	if(arg.size()==0)
		throw hException("hCeil::execute","Cannot calculate function with zero input arguments",hError);
	ret.reserve(arg.size());
	ret.push_back(arg[0].traverse((double (*)(double))std::ceil));
	return ret;
}
std::string hCeil::getHelp(void){
	return "Syntax: ceil(in)\n\nDescription: In mathematics and computer science, the floor and ceiling functions are two mathematical functions which convert arbitrary real numbers to close integers. The  ceiling function, denoted ceil(x) or ceiling(x), is the function that returns the smallest integer not less than x. For example, ceiling(2.3) = 3, ceiling(2) = 2 and ceiling(-2.3) = -2.\n\nInput parameters: 'in' can be a variable of any dimensions and any type, except for complex.\n\nExceptions: hException::hError if input argument's number is 0.\n\nReturns: a variable with the same dimensions as 'in', and of type double.\n\n";
}

hStructure hExp::execute(hStructure &arg){
	hStructure ret;
	if(arg.size()==0)
		throw hException("hExp::execute","Cannot calculate function with zero input arguments",hError);
	ret.reserve(arg.size());
	ret.push_back(arg[0].traverse((double (*)(double))std::exp));
	return ret;
}
std::string hExp::getHelp(void){
	return "Syntax: exp(in)\n\nDescription: The exponential function is a function in mathematics. The application of this function to a value x is written as exp(x). Equivalently, this can be written in the form e^x, where e is a mathematical constant, the base of the natural logarithm, which equals approximately 2.718281828, and is also known as Euler's number.In general, the variable x can be any real or complex number, or even an entirely different kind of mathematical object.\n\nInput parameters: 'in' can be a variable of any dimensions and any type, except for complex.\n\nExceptions: hException::hError if input argument's number is 0.\n\nReturns: a variable with the same dimensions as 'in', and of type double.\n\n";
}

hStructure hFloor::execute(hStructure &arg){
	hStructure ret;
	if(arg.size()==0)
		throw hException("hFloor::execute","Cannot calculate function with zero input arguments",hError);
	ret.reserve(arg.size());
	ret.push_back(arg[0].traverse((double (*)(double))std::floor));
	return ret;
}
std::string hFloor::getHelp(void){
	return "Syntax: floor(in)\n\nDescription: In mathematics and computer science, the floor and ceiling functions are two mathematical functions which convert arbitrary real numbers to close integers. The floor function of a real number x, denoted floor(x), is a function that returns the highest integer less than or equal to x. For example, floor(2.9) = 2, floor(-2) = -2 and floor(-2.3) = -3. For nonnegative x, a more traditional name for floor(x) is the integral part or integral value of x.\n\nInput parameters:'in' can be a variable of any dimensions and any type, except for complex.\n\nExceptions\n\n: hException::hError if input argument's number is 0.\n\nReturns\n\n: a variable with the same dimensions as 'in', and of type double.\n\n";
}

hStructure hLog::execute(hStructure &arg){
	hStructure ret;
	if(arg.size()==0)
		throw hException("hLog::execute","Cannot calculate function with zero input arguments",hError);
	ret.reserve(arg.size());
	ret.push_back(arg[0].traverse((double (*)(double))std::log));
	return ret;
}
std::string hLog::getHelp(void){
	return "Syntax: log(in)\n\nDescription\n\n: In mathematics, a logarithm of a given number to a given base is the power to which you need to raise the base in order to get the number. For example, the logarithm of 1000 to the common base 10 is 3, because 10 raised to a power of 3 is 1000. The function log(x) returns the logarithm of number x to the common base e, where e equals approximately 2.718281828, and is also known as Euler's number\n\nInput parameters: 'in' can be a variable of any dimensions and any type, except for complex.\n\nExceptions: hException::hError if input argument's number is 0.\n\nReturns: a variable with the same dimensions as 'in', and of type double.\n\n";
}

hStructure hLog10::execute(hStructure &arg){
	hStructure ret;
	if(arg.size()==0)
		throw hException("hLog10::execute","Cannot calculate function with zero input arguments",hError);
	ret.reserve(arg.size());
	ret.push_back(arg[0].traverse((double (*)(double))std::log10));
	return ret;
}
std::string hLog10::getHelp(void){
	return "Syntax: log10(in)\n\nDescription: In mathematics, a logarithm of a given number to a given base is the power to which you need to raise the base in order to get the number. For example, the logarithm of 1000 to the common base 10 is 3, because 10 raised to a power of 3 is 1000. The function log10(x) returns the logarithm of number x to the common base 10.\n\nInput parameters: 'in' can be a variable of any dimensions and any type, except for complex.\n\nExceptions: hException::hError if input argument's number is 0.\n\nReturns: a variable with the same dimensions as 'in', and of type double.\n\n";
}

hStructure hSqrt::execute(hStructure &arg){
	hStructure ret;
	if(arg.size()==0)
		throw hException("hSqrt::execute","Cannot calculate function with zero input arguments",hError);
	ret.reserve(arg.size());
	ret.push_back(arg[0].traverse((double (*)(double))std::sqrt));
	return ret;
}
std::string hSqrt::getHelp(void){
	return "Syntax: sqrt(in)\n\nDescription: 	In mathematics, a square root of a number x is a number r such that r^2 = x, or in words, a number r whose square (the result of multiplying the number by itself) is x. Every non-negative real number x has a unique non-negative square root, called the principal square root. For example, the principal square root of 9 is 3, denoted sqrt(9) = 3, because 3^2 = 3x3 = 9.\n\nInput parameters: 'in' can be a variable of any dimensions and any type, except for complex.\n\nExceptions: hException::hError if input argument's number is 0.\n\nReturns: a variable with the same dimensions as 'in', and of type double.\n\n";
}


