/***************************************************************************
 *   Copyright (C) 2007 by manos                                           *
 *   etsardou@users.sf.net                                                 * 
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "hStat.h"

hStat::hStat(void){
	name="stat";
	functionNames["mean"]=&statMean;	//DONE
	functionNames["stddev"]=&statStddev;	//DONE
	//functionNames["geomean"]=&statGeomean;	
	//functionNames["harmean"]=&statHarmean;
	//functionNames["sort"]=&statSort;
	//functionNames["median"]=&statMedian;
	//functionNames["mode"]=&statMode;
	//functionNames["range"]=&statRange;
	//functionNames["madev"]=&statMadev;
	functionNames["var"]=&statVar;		//DONE
	functionNames["svar"]=&statSvar;	//DONE
	functionNames["sstddev"]=&statSstddev;	//DONE
	//functionNames["coevar"]=&statCoevar;
	//functionNames["assym1"]=&statAssym1;
	//functionNames["assym2"]=&statAssym2;
	//functionNames["assym3"]=&statAssym3;
	//functionNames["ropi"]=&statRopi;
	//functionNames["gassym1"]=&statGassym1;
	//functionNames["gassym2"]=&statGassym2;
	//functionNames["bgpline"]=&statBgpline;
	//functionNames["agpline"]=&statAgpline;
	//functionNames["corr"]=&statCorr;
}

std::map<std::string,hAbstractFunction *> hStat::getNames(void){
	return functionNames;
}

hStructure hCorr::execute(hStructure &arg){
	if(arg.size()!=inArgs)
		throw hException("hMean::execute","Invalid number of input arguments",hBadArg);
	hVariable in=arg[0];
	if(in.getDim()!=2)
		throw hException("hMean::execute","Table must have 2 dimensions",hBadArg);
	unsigned int *i;
	i=in.getSize();
	hVariable S;
	unsigned int index[2];
	if(i[0]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[1]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[1];
	}else if(i[1]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[0]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[0];
	}else{
		throw hException("hMean::execute","Table's first or second dimension must be of size 1",hBadArg);
	}
	hStructure r;
	r.push_back(S);
	return r;
}

hStructure hAgpline::execute(hStructure &arg){
	if(arg.size()!=inArgs)
		throw hException("hMean::execute","Invalid number of input arguments",hBadArg);
	hVariable in=arg[0];
	if(in.getDim()!=2)
		throw hException("hMean::execute","Table must have 2 dimensions",hBadArg);
	unsigned int *i;
	i=in.getSize();
	hVariable S;
	unsigned int index[2];
	if(i[0]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[1]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[1];
	}else if(i[1]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[0]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[0];
	}else{
		throw hException("hMean::execute","Table's first or second dimension must be of size 1",hBadArg);
	}
	hStructure r;
	r.push_back(S);
	return r;
}

hStructure hBgpline::execute(hStructure &arg){
	if(arg.size()!=inArgs)
		throw hException("hMean::execute","Invalid number of input arguments",hBadArg);
	hVariable in=arg[0];
	if(in.getDim()!=2)
		throw hException("hMean::execute","Table must have 2 dimensions",hBadArg);
	unsigned int *i;
	i=in.getSize();
	hVariable S;
	unsigned int index[2];
	if(i[0]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[1]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[1];
	}else if(i[1]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[0]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[0];
	}else{
		throw hException("hMean::execute","Table's first or second dimension must be of size 1",hBadArg);
	}
	hStructure r;
	r.push_back(S);
	return r;
}

hStructure hGassym2::execute(hStructure &arg){
	if(arg.size()!=inArgs)
		throw hException("hMean::execute","Invalid number of input arguments",hBadArg);
	hVariable in=arg[0];
	if(in.getDim()!=2)
		throw hException("hMean::execute","Table must have 2 dimensions",hBadArg);
	unsigned int *i;
	i=in.getSize();
	hVariable S;
	unsigned int index[2];
	if(i[0]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[1]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[1];
	}else if(i[1]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[0]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[0];
	}else{
		throw hException("hMean::execute","Table's first or second dimension must be of size 1",hBadArg);
	}
	hStructure r;
	r.push_back(S);
	return r;
}

hStructure hGassym1::execute(hStructure &arg){
	if(arg.size()!=inArgs)
		throw hException("hMean::execute","Invalid number of input arguments",hBadArg);
	hVariable in=arg[0];
	if(in.getDim()!=2)
		throw hException("hMean::execute","Table must have 2 dimensions",hBadArg);
	unsigned int *i;
	i=in.getSize();
	hVariable S;
	unsigned int index[2];
	if(i[0]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[1]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[1];
	}else if(i[1]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[0]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[0];
	}else{
		throw hException("hMean::execute","Table's first or second dimension must be of size 1",hBadArg);
	}
	hStructure r;
	r.push_back(S);
	return r;
}

hStructure hRopi::execute(hStructure &arg){
	if(arg.size()!=inArgs)
		throw hException("hMean::execute","Invalid number of input arguments",hBadArg);
	hVariable in=arg[0];
	if(in.getDim()!=2)
		throw hException("hMean::execute","Table must have 2 dimensions",hBadArg);
	unsigned int *i;
	i=in.getSize();
	hVariable S;
	unsigned int index[2];
	if(i[0]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[1]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[1];
	}else if(i[1]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[0]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[0];
	}else{
		throw hException("hMean::execute","Table's first or second dimension must be of size 1",hBadArg);
	}
	hStructure r;
	r.push_back(S);
	return r;
}

hStructure hAssym3::execute(hStructure &arg){
	if(arg.size()!=inArgs)
		throw hException("hMean::execute","Invalid number of input arguments",hBadArg);
	hVariable in=arg[0];
	if(in.getDim()!=2)
		throw hException("hMean::execute","Table must have 2 dimensions",hBadArg);
	unsigned int *i;
	i=in.getSize();
	hVariable S;
	unsigned int index[2];
	if(i[0]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[1]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[1];
	}else if(i[1]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[0]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[0];
	}else{
		throw hException("hMean::execute","Table's first or second dimension must be of size 1",hBadArg);
	}
	hStructure r;
	r.push_back(S);
	return r;
}

hStructure hAssym2::execute(hStructure &arg){
	if(arg.size()!=inArgs)
		throw hException("hMean::execute","Invalid number of input arguments",hBadArg);
	hVariable in=arg[0];
	if(in.getDim()!=2)
		throw hException("hMean::execute","Table must have 2 dimensions",hBadArg);
	unsigned int *i;
	i=in.getSize();
	hVariable S;
	unsigned int index[2];
	if(i[0]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[1]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[1];
	}else if(i[1]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[0]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[0];
	}else{
		throw hException("hMean::execute","Table's first or second dimension must be of size 1",hBadArg);
	}
	hStructure r;
	r.push_back(S);
	return r;
}

hStructure hAssym1::execute(hStructure &arg){
	if(arg.size()!=inArgs)
		throw hException("hMean::execute","Invalid number of input arguments",hBadArg);
	hVariable in=arg[0];
	if(in.getDim()!=2)
		throw hException("hMean::execute","Table must have 2 dimensions",hBadArg);
	unsigned int *i;
	i=in.getSize();
	hVariable S;
	unsigned int index[2];
	if(i[0]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[1]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[1];
	}else if(i[1]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[0]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[0];
	}else{
		throw hException("hMean::execute","Table's first or second dimension must be of size 1",hBadArg);
	}
	hStructure r;
	r.push_back(S);
	return r;
}

hStructure hCoevar::execute(hStructure &arg){
	hMean mean; hSstddev sstddev;
	hStructure temp1=mean.execute(arg);
	hStructure temp2=sstddev.execute(arg);
	temp2[0]=temp2[0]/temp1[0];
	return temp2;
}

hStructure hSstddev::execute(hStructure &arg){
	hSvar svar; hSqrt sqrt;
	hStructure temp=svar.execute(arg);
	return sqrt.execute(temp);
}
std::string hSstddev::getHelp(void){
	return "Syntax: sstddev(a)\n\nDescription: Function sstddev stands for the 'sample standard deviation' of an array of values.\n\nInput parameters: 'a' can be either a row vector or a column vector.\n\nExceptions: \n\n\thException::hError if input is not a row or column vector.\n\nReturns: a number of type double, containing the sample standard deviation of a.\n\n";
}

hStructure hSvar::execute(hStructure &arg){
	if(arg.size()!=inArgs)
		throw hException("hSvar::execute","Invalid number of input arguments",hBadArg);
	hVariable in=arg[0];
	if(in.getDim()!=2)
		throw hException("hSvar::execute","Table must have 2 dimensions",hBadArg);
	unsigned int *i;
	i=in.getSize();
	hVariable S;
	unsigned int index[2];
	if(i[0]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0])*in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[1]=j;
			S=S+in.gethVariable(&index[0])*in.gethVariable(&index[0]);
		}
		hMean mean; hStructure temp;
		temp=mean.execute(arg);
		S=S.convert<float>()/((float)(i[1]-1))-temp[0]*temp[0]*((float)i[1]/(float)(i[1]-1));
	}else if(i[1]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0])*in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[0];++j){
			index[0]=j;
			S=S+in.gethVariable(&index[0])*in.gethVariable(&index[0]);
		}
		hMean mean; hStructure temp;
		temp=mean.execute(arg);
		S=S.convert<float>()/((float)(i[0]-1))-temp[0]*temp[0]*((float)i[0]/(float)(i[0]-1));
	}else{
		throw hException("hSvar::execute","Table's first or second dimension must be of size 1",hBadArg);
	}
	hStructure r;
	r.push_back(S);
	return r;
}
std::string hSvar::getHelp(void){
	return "Syntax: svar(a)\n\nDescription: Function svar stands for the 'sample variance' of an array of values.\n\nInput parameters: 'a' can be either a row vector or a column vector.\n\nExceptions:\n\n\thException::hError if input is not a row or column vector.\n\nReturns: a number of type double, containing the sample variance of a.\n\n";
}

hStructure hVar::execute(hStructure &arg){
	if(arg.size()!=inArgs)
		throw hException("hVar::execute","Invalid number of input arguments",hBadArg);
	hVariable in=arg[0];
	if(in.getDim()!=2)
		throw hException("hVar::execute","Table must have 2 dimensions",hBadArg);
	unsigned int *i;
	i=in.getSize();
	hVariable S;
	unsigned int index[2];
	if(i[0]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0])*in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[1]=j;
			S=S+in.gethVariable(&index[0])*in.gethVariable(&index[0]);
		}
		hMean mean; hStructure temp;
		temp=mean.execute(arg);
		S=S.convert<float>()/(float)i[1]-temp[0]*temp[0];
	}else if(i[1]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0])*in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[0];++j){
			index[0]=j;
			S=S+in.gethVariable(&index[0])*in.gethVariable(&index[0]);
		}
		hMean mean; hStructure temp;
		temp=mean.execute(arg);
		S=S.convert<float>()/(float)i[0]-temp[0]*temp[0];
	}else{
		throw hException("hVar::execute","Table's first or second dimension must be of size 1",hBadArg);
	}
	hStructure r;
	r.push_back(S);
	return r;
}
std::string hVar::getHelp(void){
	return "Syntax: var(a)\n\nDescription: Function var stands for the 'variance' of an array of values.\n\nInput parameters: 'a' can be either a row vector or a column vector.\n\nExceptions:\n\n\thException::hError if input is not a row or column vector.\n\nReturns: a number of type double, containing the variance of a.\n\n";
}

hStructure hMadev::execute(hStructure &arg){
	if(arg.size()!=inArgs)
		throw hException("hMean::execute","Invalid number of input arguments",hBadArg);
	hVariable in=arg[0];
	if(in.getDim()!=2)
		throw hException("hMean::execute","Table must have 2 dimensions",hBadArg);
	unsigned int *i;
	i=in.getSize();
	hVariable S;
	unsigned int index[2];
	if(i[0]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[1]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[1];
	}else if(i[1]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[0]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[0];
	}else{
		throw hException("hMean::execute","Table's first or second dimension must be of size 1",hBadArg);
	}
	hStructure r;
	r.push_back(S);
	return r;
}

hStructure hRange::execute(hStructure &arg){
	if(arg.size()!=inArgs)
		throw hException("hMean::execute","Invalid number of input arguments",hBadArg);
	hVariable in=arg[0];
	if(in.getDim()!=2)
		throw hException("hMean::execute","Table must have 2 dimensions",hBadArg);
	unsigned int *i;
	i=in.getSize();
	hVariable S;
	unsigned int index[2];
	if(i[0]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[1]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[1];
	}else if(i[1]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[0]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[0];
	}else{
		throw hException("hMean::execute","Table's first or second dimension must be of size 1",hBadArg);
	}
	hStructure r;
	r.push_back(S);
	return r;
}

hStructure hMode::execute(hStructure &arg){
	if(arg.size()!=inArgs)
		throw hException("hMean::execute","Invalid number of input arguments",hBadArg);
	hVariable in=arg[0];
	if(in.getDim()!=2)
		throw hException("hMean::execute","Table must have 2 dimensions",hBadArg);
	unsigned int *i;
	i=in.getSize();
	hVariable S;
	unsigned int index[2];
	if(i[0]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[1]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[1];
	}else if(i[1]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[0]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[0];
	}else{
		throw hException("hMean::execute","Table's first or second dimension must be of size 1",hBadArg);
	}
	hStructure r;
	r.push_back(S);
	return r;
}

hStructure hMedian::execute(hStructure &arg){
	if(arg.size()!=inArgs)
		throw hException("hMean::execute","Invalid number of input arguments",hBadArg);
	hVariable in=arg[0];
	if(in.getDim()!=2)
		throw hException("hMean::execute","Table must have 2 dimensions",hBadArg);
	unsigned int *i;
	i=in.getSize();
	hVariable S;
	unsigned int index[2];
	if(i[0]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[1]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[1];
	}else if(i[1]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[0]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[0];
	}else{
		throw hException("hMean::execute","Table's first or second dimension must be of size 1",hBadArg);
	}
	hStructure r;
	r.push_back(S);
	return r;
}

hStructure hSort::execute(hStructure &arg){
	if(arg.size()!=inArgs)
		throw hException("hMean::execute","Invalid number of input arguments",hBadArg);
	hVariable in=arg[0];
	if(in.getDim()!=2)
		throw hException("hMean::execute","Table must have 2 dimensions",hBadArg);
	unsigned int *i;
	i=in.getSize();
	hVariable S;
	unsigned int index[2];
	if(i[0]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[1]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[1];
	}else if(i[1]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[0]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[0];
	}else{
		throw hException("hMean::execute","Table's first or second dimension must be of size 1",hBadArg);
	}
	hStructure r;
	r.push_back(S);
	return r;
}

hStructure hHarmean::execute(hStructure &arg){
	if(arg.size()!=inArgs)
		throw hException("hMean::execute","Invalid number of input arguments",hBadArg);
	hVariable in=arg[0];
	if(in.getDim()!=2)
		throw hException("hMean::execute","Table must have 2 dimensions",hBadArg);
	unsigned int *i;
	i=in.getSize();
	hVariable S;
	unsigned int index[2];
	if(i[0]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[1]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[1];
	}else if(i[1]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[0]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[0];
	}else{
		throw hException("hMean::execute","Table's first or second dimension must be of size 1",hBadArg);
	}
	hStructure r;
	r.push_back(S);
	return r;
}

hStructure hGeomean::execute(hStructure &arg){
	if(arg.size()!=inArgs)
		throw hException("hMean::execute","Invalid number of input arguments",hBadArg);
	hVariable in=arg[0];
	if(in.getDim()!=2)
		throw hException("hMean::execute","Table must have 2 dimensions",hBadArg);
	unsigned int *i;
	i=in.getSize();
	hVariable S;
	unsigned int index[2];
	if(i[0]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[1]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[1];
	}else if(i[1]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[0]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[0];
	}else{
		throw hException("hMean::execute","Table's first or second dimension must be of size 1",hBadArg);
	}
	hStructure r;
	r.push_back(S);
	return r;
}

hStructure hMean::execute(hStructure &arg){
	if(arg.size()!=inArgs)
		throw hException("hMean::execute","Invalid number of input arguments",hBadArg);
	hVariable in=arg[0];
	if(in.getDim()!=2)
		throw hException("hMean::execute","Table must have 2 dimensions",hBadArg);
	unsigned int *i;
	i=in.getSize();
	hVariable S;
	unsigned int index[2];
	if(i[0]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[1];++j){
			index[1]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[1];
	}else if(i[1]==1){
		index[0]=0;
		index[1]=0;
		S=in.gethVariable(&index[0]);
		for(unsigned int j=1;j<i[0];++j){
			index[0]=j;
			S=S+in.gethVariable(&index[0]);
		}
		S=S.convert<float>()/(float)i[0];
	}else{
		throw hException("hMean::execute","Table's first or second dimension must be of size 1\n\n",hBadArg);
	}
	hStructure r;
	r.push_back(S);
	return r;
}
std::string hMean::getHelp(void){
	return "Syntax: mean(a)\n\nDescription: Function mean stands for the 'arithmetic mean' of an array of values. It must have one argument which must be either a row vector or a column vector.\n\nInput parameters: 'a' can be either a row vector or a column vector.\n\nExceptions:\n\n\thException::hError if input is not a row or column vector.\n\nReturns: a number of type double, containing the arithmetic mean of a.\n\n";
}

hStructure hStddev::execute(hStructure &arg){
	hVar var; hSqrt sqrt;
	hStructure temp=var.execute(arg);
	return sqrt.execute(temp);
}
std::string hStddev::getHelp(void){
	return "Syntax: stddev(a)\n\nDescription: Function stddev stands for the 'standard deviation' of an array of values.\n\nInput parameters: 'a' can be either a row vector or a column vector.\n\nExceptions:\n\n\thException::hError if input is not a row or column vector.\n\nReturns: a number of type double, containing the standard deviation of a.\n\n";
}
