/***************************************************************************
 *   Copyright (C) 2007 by Harry Serenis                                   *
 *   bobomastoras@users.sf.net                                             * 
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef HSYSTEMCOMMAND_H
#define HSYSTEMCOMMAND_H

#include <string>
#include <vector>
#include <iostream>
#include "hException.h"

template <class handlerType>
class hCmds{
protected:
	std::vector<std::string> command;
	std::vector<int (*)(handlerType*,std::vector<std::string>)> cmdHandler;

void clear(void){
	command.clear();
	cmdHandler.clear();
}
public:


void addCommand(std::string cmd,int (*handlerID)(handlerType*,std::vector<std::string>)){
	command.push_back(cmd);
	cmdHandler.push_back(handlerID);
}


};

template <class handlerType>
class hSystemCommand:public hCmds<handlerType>{
	std::vector<std::string> argv;
	bool valid;
	int (*handler)(handlerType*,std::vector<std::string>);
	handlerType *obj;
public:

std::vector<std::string> getCommandNames(void){
	return hCmds<handlerType>::command;
}

hSystemCommand(){
	obj=NULL;
}
	
hSystemCommand(handlerType *objHandler){
	obj=objHandler;
	valid=false;
}
	

void setCommand(std::string input){
	unsigned int i,j=0;
	argv.clear();
	i=input.find(' ' ,0);
	while(i!=-1){
		argv.push_back(input.substr(j,i-j));
		i=input.find(' ',j=i+1);
	}
	argv.push_back(input.substr(j,input.size()-j));
	valid=0;
	for(i=0;i<hCmds<handlerType>::command.size();i++){
		if(!argv.at(0).compare(hCmds<handlerType>::command.at(i))){
			valid=true;
			handler=hCmds<handlerType>::cmdHandler.at(i);
		}
	}
}

bool isValid(void){
	return valid;
}

int execute(void){
	if(obj==NULL)
		throw hException("hSystemCommands::execute","Handler not linked to object",hBadCmd);
	if(valid==false)
		throw hException("hSystemCommands::execute","Invalid Command",hBadCmd);
	int (*f)(handlerType*,std::vector<std::string>);
	f=handler;
	return f(obj,argv);
}

void clear(void){
	argv.clear();
	valid=false;
}


};

#endif

