/***************************************************************************
 *   Copyright (C) 2007 by aurelius                                        *
 *   sg_aurelius@users.sf.net                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "hTools.h"

void hSymbol::set_symbol(string str, int pow)
{
   sym = str ;
   power = pow ;
}

string hSymbol::get_symbol_expression()
{
   return sym ;
}

int hSymbol::get_symbol_power()
{
   return power ;
}

/**
   In order to have compatibility with C++ format, the precedence of operators are same as in C++.
   http://en.wikipedia.org/wiki/Order_of_operations
*/

int hTools::symbol_number = _SYMBOL_NUMBER;

hSymbol hTools::symbols[_SYMBOL_NUMBER]={hSymbol("+", 4),hSymbol("-", 4),hSymbol("*", 3),hSymbol(".*", 3),
			hSymbol("/", 3),hSymbol("./", 3),hSymbol("%", 3),hSymbol(".%", 3),
			hSymbol("^", 3),hSymbol(".^", 3),hSymbol("<", 6),hSymbol(">", 6),
			hSymbol("<=", 6),hSymbol(">=", 6),hSymbol("==", 7),hSymbol("!=", 7),
			hSymbol("&", 8),hSymbol("|", 10),hSymbol("&&", 12),hSymbol("||", 12),
			hSymbol("!", 2), hSymbol("**", 3), hSymbol("++", 2), hSymbol("--", 2)};

hTools::hTools()
{
  /* symbol_number = 21 ;
   symbols = new hSymbol[symbol_number] ;
   
   symbols[0].set_symbol("+", 4);
   symbols[1].set_symbol("-", 4);
   symbols[2].set_symbol("*", 3);
   symbols[3].set_symbol(".*", 3);
   symbols[4].set_symbol("/", 3);
   symbols[5].set_symbol("./", 3);
   symbols[6].set_symbol("%", 3);
   symbols[7].set_symbol(".%", 3);
   symbols[8].set_symbol("^", 3);
   symbols[9].set_symbol(".^", 3);
   symbols[10].set_symbol("<", 6);
   symbols[11].set_symbol(">", 6);
   symbols[12].set_symbol("<=", 6);
   symbols[13].set_symbol(">=", 6);
   symbols[14].set_symbol("==", 7);
   symbols[15].set_symbol("!=", 7);
   symbols[16].set_symbol("&", 8);
   symbols[17].set_symbol("|", 10);
   symbols[18].set_symbol("&&", 11);
   symbols[19].set_symbol("||", 12);
   symbols[19].set_symbol("!", 2);*/
}

hSymbol* hTools::getSymbols()
{
   return symbols ;
}

int hTools::getSymbolNumber()
{
   return symbol_number ;
}

int hTools::getSymbolPower(string str)
{
   hSymbol *syms = getSymbols() ;
   int i, num = getSymbolNumber() ;
   
   for(i = 0 ; i < num ; i++)
   {
	  if(str == syms[i].get_symbol_expression())
		 return syms[i].get_symbol_power() ;
   }
   
   return -1 ;
}

/**
   The function same type alerts us, if the two strings are of same type or not, in order
   to tokenize input. if they are of the same type, the result is -1 and we have to move on
   to the next character. The other results are till now:
   number - 0
   function or matrix - 1
   matrix - 2
   argument separator - 3
   one symbol operator - 4
   two symbol operator - 5
   left parenthesis - 6
   right parenthesis - 7
   variable -8          NOTE:: we do not examine this case in tihs function
   matrix to create - 9
   left bracket - 10
   right bracket - 11
   char variable - 12

   Of course, in the future, some more results can be added, such as various symbols, strings
   etc.
*/

int hTools::same_type(string tok, string tok_1)
{
   if(isSymbol(tok))
   {
	   return one_sym_oper ;
   }
   else if(isNumber(tok_1))
   {
	  if(isNumber(tok)) return -1 ;
	  else return number;
   }
   else if(isAlpharithmetic(tok_1))
   {
	  if(isAlpharithmetic(tok)) return -1 ;
	  else return func_or_matr ;
   }
   else if(tok_1 == ",")
   {
	   return arg_separ ;
   }
   else if(isSymbol(tok_1) && isSymbol(tok.substr(tok.length() - 1)) && !isSymbol(tok) && tok[1] != '!')
   {
	  throw hException("hTools::same_type", "Invalid expression. Be careful!!! There is no such thing as " + tok +" !!!", hInvalidExpression) ;
   }
   else if(isSymbol(tok_1))
   {
	   return two_sym_oper ;
   }
   else if(tok_1 == "(")
   {
	   return left_par ;
   }
   else if(tok_1 == ")")
   {
	   return right_par ;
   }
   else if(tok_1 == "[")
   {
	   return left_brack ;
   }
   else if(tok_1 == "]")
   {
	   return right_brack ;
   }
   else
   {
	   return -2 ;
   }
   /*
   matrix here
   */
}

bool hTools::is_valid_char(char ch)
{
   if((0 <= ch && ch <= 34) || (37 <= ch && ch <= 38) || ( 40 <= ch  && ch <= 57) || (60 <= ch && ch <= 62) || (65 <= ch && ch <= 91) || ( 93 <= ch && ch <= 95) || (96 <= ch && ch <= 125))
      return true ;
   else
	  return false ;
}

bool hTools::isSymbol(string str)
{
   int i, num = getSymbolNumber();
   //if(str == "") throw hException("hTools::isSymbol", "Program malfuntion.", hError) ;
   for(i = 0 ; i < num ; i++)
   {
	  if(str == symbols[i].get_symbol_expression())
		 return true ;
   }
   return false ;
}

bool hTools::isAlpharithmetic(string str)
{
   int i;
   
   for(i = 0 ; i < str.length() ; i++)
	  if( !( (48 <= str[i] && str[i] <= 57) ||  (65 <= str[i] && str[i] <= 90) || (97 <= str[i] && str[i] <= 122) || str[i] == 95 ) ) return false;
   
   return true ;
}

bool hTools::isNumber(string str)
{
   int i;
   
   for(i = 0 ; i < str.length() ; i++)
   {
	  if((str[i] == 45 && i!= 0) || !((46 <= str[i] && str[i] <= 57) && str[i] != 47) ) return false;
   }
   
   return true ;
}

bool hTools::isNumber(char in)
{
	if(in=='.' || in=='0'||in=='1'||in=='2'||in=='3'||in=='4'||in=='5'||in=='6'||in=='7'||in=='8'||in=='9') return true;
	else return false;
}

bool hTools::isLetter(char in)
{
	if((in>=65 && in<=90) || (in>=97 && in<=122)) return true;
	else return false;
}

int hTools::isSymbol(char ch1, char ch2, string *symbols, unsigned int siz, unsigned int repeat)
{
	string s1, s2;
	char *temp;
	try
	{
		temp=new char[3];
	}
	catch(std::exception &e)
	{
		throw hException("hTools::isSymbol ", "There is no memory", hAlloc);
	}
	temp[0]=ch1;temp[1]=ch2;temp[2]='\0';
	s1=(char)ch1;
	s2=string(temp);
	delete [] temp;
	if(s2==symbols[repeat]) return repeat;
	if(s1==symbols[repeat]) return repeat;
	return -1;
}

int hTools::isSymbol(char ch1, char ch2, string *symbols, unsigned int siz)
{
	int i=siz-1;
	char *temp;
	try
	{
		temp=new char[3];
	}
	catch(std::exception &e)
	{
		throw hException("hTools::isSymbol ", "There is no memory", hAlloc);
	}
	temp[0]=ch1;temp[1]=ch2;temp[2]='\0';
	string st1, st2;
	st1=(char)ch1;
	st2=string(temp);
	delete [] temp;
	for(i=siz-1;i>-1;i--)
	{
		if(st2==symbols[i]) return i;
		if(st1==symbols[i]) return i;
	}	
	return -1;
}

bool hTools::containsLetter(string num)
{
	for(int i=0;i<num.length();i++) if(isLetter(num[i])) return true;
	return false;
}

int hTools::numberContainsValidChars(string num)
{
	int temp=0;
	if(num == "") return 0;
	for(int i=0;i<num.length();i++)
	{
	   if(i == 0 && num[i] == '-')
	   {
	      continue;
	   }
	   if(!isNumber(num[i]) && num[i]!='.') return 0;
	   else if(num[i]=='.') temp++;
	}
	if(temp>1) return 0;
	else return 1;
}

int hTools::findSymbol(string expression, int &pos, string *symbols, unsigned int siz)
{
	int ret=-1;
	unsigned int tempPos=pos;
	for(int i=siz-1;i>=0;i--)
	{
		tempPos=expression.find(symbols[i]);
		if(tempPos!=string::npos)
		{
			ret=i;pos=tempPos;
		}
	}
	return ret;
}

string hTools::invertStringForDivision(string str)
{
   int i, length = str.length(), position1 = 0, position2 = length;
   string ret_str = "", temp1 = str, temp2 = str+'/';
   
   while(temp2.find('/') != string::npos)
   {
      position2 = temp2.find('/');
	  temp1 = temp2.substr(0, position2) ;
	  temp2 = temp2.substr(position2 + 1) ;
	  position1 = position2 + 1 ;
	  ret_str = '/' + temp1 + ret_str ;
   }
   
   ret_str = ret_str.substr(1);
   
   return ret_str ;
}


int hTools::same_type(string tok, string tok_1, std::string &error, int &error_code)
{
	if(isSymbol(tok))
	{
		return 4 ;
	}
	else if(isNumber(tok_1))
	{
		if(isNumber(tok)) return -1 ;
		else return 0;
	}
	else if(isAlpharithmetic(tok_1))
	{
		if(isAlpharithmetic(tok)) return -1 ;
		else return 1 ;
	}
	else if(tok_1 == ",")
	{
		return 3 ;
	}
	else if(isSymbol(tok_1) && isSymbol(tok.substr(tok.length() - 1)) && !isSymbol(tok) && tok[1] != '!')
	{
		//throw hException("hTools::same_type", "Invalid expression. Be careful!!! There is no such thing as " + tok +" !!!", hInvalidExpression) ;
		if(error.empty())
		{
			error = "Invalid expression. Be careful!!! There is no such thing as " + tok +" !!!" ;
			error_code = 1 ;
		}
		return -2 ;
	}
	else if(isSymbol(tok_1))
	{
		return 5 ;
	}
	else if(tok_1 == "(")
	{
		return 6 ;
	}
	else if(tok_1 == ")")
	{
		return 7 ;
	}
	else if(tok_1 == "[")
	{
		return 10 ;
	}
	else if(tok_1 == "]")
	{
		return 11 ;
	}
	else
	{
		return -2 ;
	}
   /*
	matrix here
   */
}
