/***************************************************************************
 *   Copyright (C) 2008 by Bob,,,   *
 *   bob@lab17   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _HVARBASE_H
#define _HVARBASE_H

#include <complex>
#include <vector>

#include "hVarType.h"
#include "hException.h"

namespace hvar{
enum allocation{
	linear=0
};
}

class hVarBase{
public:
	virtual hVarBase * copy(void)=0;
	virtual hvar::allocation allocationMethod(void)=0;
	virtual unsigned int getDim(void)=0;
	virtual std::vector<unsigned int> getSize(void)=0;
	virtual unsigned int getSize(unsigned int dimension)=0;
	virtual void setValue(std::vector<unsigned int> index,hVarBase* data)=0;
	virtual void setValue(unsigned long *index,hVarBase* data)=0;
	virtual void setValue(hVarBase* data)=0;

	virtual void setValue(std::vector<unsigned int> index,const bool &data)=0;
	virtual void setValue(std::vector<unsigned int> index,const unsigned char &data)=0;
	virtual void setValue(std::vector<unsigned int> index,const signed char &data)=0;
	virtual void setValue(std::vector<unsigned int> index,const unsigned short &data)=0;
	virtual void setValue(std::vector<unsigned int> index,const signed short &data)=0;
	virtual void setValue(std::vector<unsigned int> index,const unsigned long &data)=0;
	virtual void setValue(std::vector<unsigned int> index,const signed long &data)=0;
//	virtual void setValue(std::vector<unsigned int> index,const unsigned int &data)=0;
//	virtual void setValue(std::vector<unsigned int> index,const signed int &data)=0;
	virtual void setValue(std::vector<unsigned int> index,const float &data)=0;
	virtual void setValue(std::vector<unsigned int> index,const double &data)=0;
	virtual void setValue(std::vector<unsigned int> index,const long double &data)=0;
	virtual void setValue(std::vector<unsigned int> index,const std::complex<float> &data)=0;
	virtual void setValue(std::vector<unsigned int> index,const std::complex<double> &data)=0;
	virtual void setValue(std::vector<unsigned int> index,const std::complex<long double> &data)=0;

	virtual void setValue(const bool &data)=0;
	virtual void setValue(const unsigned char &data)=0;
	virtual void setValue(const signed char &data)=0;
	virtual void setValue(const unsigned short &data)=0;
	virtual void setValue(const signed short &data)=0;
	virtual void setValue(const unsigned long &data)=0;
	virtual void setValue(const signed long &data)=0;
//	virtual void setValue(const unsigned int &data)=0;
//	virtual void setValue(const signed int &data)=0;
	virtual void setValue(const float &data)=0;
	virtual void setValue(const double &data)=0;
	virtual void setValue(const long double &data)=0;
	virtual void setValue(const std::complex<float> &data)=0;
	virtual void setValue(const std::complex<double> &data)=0;
	virtual void setValue(const std::complex<long double> &data)=0;

	virtual hVarBase* getValue(std::vector<unsigned int> index)=0;
	virtual hVarBase* getValue(void)=0;
	virtual bool getValue(bool &data)=0;
	virtual unsigned char getValue(unsigned char &data)=0;
	virtual signed char getValue(signed char &data)=0;
	virtual unsigned short getValue(unsigned short &data)=0;
	virtual signed short getValue(signed short &data)=0;
	virtual unsigned long getValue(unsigned long &data)=0;
	virtual signed long getValue(signed long &data)=0;
	virtual float getValue(float &data)=0;
	virtual double getValue(double &data)=0;
	virtual long double getValue(long double &data)=0;
	virtual std::complex<float> getValue(std::complex<float> &data)=0;
	virtual std::complex<double> getValue(std::complex<double> &data)=0;
	virtual std::complex<long double> getValue(std::complex<long double> &data)=0;

	virtual bool getValue(std::vector<unsigned int> index,bool &data)=0;
	virtual unsigned char getValue(std::vector<unsigned int> index,unsigned char &data)=0;
	virtual signed char getValue(std::vector<unsigned int> index,signed char &data)=0;
	virtual unsigned short getValue(std::vector<unsigned int> index,unsigned short &data)=0;
	virtual signed short getValue(std::vector<unsigned int> index,signed short &data)=0;
	virtual unsigned long getValue(std::vector<unsigned int> index,unsigned long &data)=0;
	virtual signed long getValue(std::vector<unsigned int> index,signed long &data)=0;
	virtual float getValue(std::vector<unsigned int> index,float &data)=0;
	virtual double getValue(std::vector<unsigned int> index,double &data)=0;
	virtual long double getValue(std::vector<unsigned int> index,long double &data)=0;
	virtual std::complex<float> getValue(std::vector<unsigned int> index,std::complex<float> &data)=0;
	virtual std::complex<double> getValue(std::vector<unsigned int> index,std::complex<double> &data)=0;
	virtual std::complex<long double> getValue(std::vector<unsigned int> index,std::complex<long double> &data)=0;
	virtual void get(bool *data)=0;
	virtual void get(unsigned char *data)=0;
	virtual void get(signed char *data)=0;
	virtual void get(unsigned short *data)=0;
	virtual void get(signed short *data)=0;
	virtual void get(unsigned long *data)=0;
	virtual void get(signed long *data)=0;
	virtual void get(float *data)=0;
	virtual void get(double *data)=0;
	virtual void get(long double *data)=0;
	virtual void get(std::complex<float> *data)=0;
	virtual void get(std::complex<double> *data)=0;
	virtual void get(std::complex<long double> *data)=0;

	virtual hVarBase * matrixMul(hVarBase *B)=0;

	virtual unsigned char getType(void)=0;

	virtual void clear(void)=0;

	virtual hVarBase* cast(unsigned char type)=0;
	
	virtual hVarBase* convert(bool flank)=0;
	virtual hVarBase* convert(unsigned char flank)=0;
	virtual hVarBase* convert(signed char flank)=0;
	virtual hVarBase* convert(unsigned short flank)=0;
	virtual hVarBase* convert(signed short flank)=0;
	virtual hVarBase* convert(unsigned long flank)=0;
	virtual hVarBase* convert(signed long flank)=0;
//	virtual hVarBase* convert(unsigned int flank)=0;
//	virtual hVarBase* convert(signed int flank)=0;
	virtual hVarBase* convert(float flank)=0;
	virtual hVarBase* convert(double flank)=0;
	virtual hVarBase* convert(long double flank)=0;
	virtual hVarBase* convert(std::complex<float> flank)=0;
	virtual hVarBase* convert(std::complex<double> flank)=0;
	virtual hVarBase* convert(std::complex<long double> flank)=0;

	virtual hVarBase* operator+(const bool &op2);
	virtual hVarBase* operator+(const unsigned char &op2);
	virtual hVarBase* operator+(const signed char &op2);
	virtual hVarBase* operator+(const unsigned short &op2);
	virtual hVarBase* operator+(const signed short &op2);
	virtual hVarBase* operator+(const unsigned long &op2);
	virtual hVarBase* operator+(const signed long &op2);
	virtual hVarBase* operator+(const float &op2);
	virtual hVarBase* operator+(const double &op2);
	virtual hVarBase* operator+(const long double &op2);
	virtual hVarBase* operator+(const std::complex<float> &op2);
	virtual hVarBase* operator+(const std::complex<double> &op2);
	virtual hVarBase* operator+(const std::complex<long double> &op2);
	virtual hVarBase* operator-(const bool &op2);
	virtual hVarBase* operator-(const unsigned char &op2);
	virtual hVarBase* operator-(const signed char &op2);
	virtual hVarBase* operator-(const unsigned short &op2);
	virtual hVarBase* operator-(const signed short &op2);
	virtual hVarBase* operator-(const unsigned long &op2);
	virtual hVarBase* operator-(const signed long &op2);
	virtual hVarBase* operator-(const float &op2);
	virtual hVarBase* operator-(const double &op2);
	virtual hVarBase* operator-(const long double &op2);
	virtual hVarBase* operator-(const std::complex<float> &op2);
	virtual hVarBase* operator-(const std::complex<double> &op2);
	virtual hVarBase* operator-(const std::complex<long double> &op2);
	virtual hVarBase* operator*(const bool &op2);
	virtual hVarBase* operator*(const unsigned char &op2);
	virtual hVarBase* operator*(const signed char &op2);
	virtual hVarBase* operator*(const unsigned short &op2);
	virtual hVarBase* operator*(const signed short &op2);
	virtual hVarBase* operator*(const unsigned long &op2);
	virtual hVarBase* operator*(const signed long &op2);
	virtual hVarBase* operator*(const float &op2);
	virtual hVarBase* operator*(const double &op2);
	virtual hVarBase* operator*(const long double &op2);
	virtual hVarBase* operator*(const std::complex<float> &op2);
	virtual hVarBase* operator*(const std::complex<double> &op2);
	virtual hVarBase* operator*(const std::complex<long double> &op2);
	virtual hVarBase* operator/(const bool &op2);
	virtual hVarBase* operator/(const unsigned char &op2);
	virtual hVarBase* operator/(const signed char &op2);
	virtual hVarBase* operator/(const unsigned short &op2);
	virtual hVarBase* operator/(const signed short &op2);
	virtual hVarBase* operator/(const unsigned long &op2);
	virtual hVarBase* operator/(const signed long &op2);
	virtual hVarBase* operator/(const float &op2);
	virtual hVarBase* operator/(const double &op2);
	virtual hVarBase* operator/(const long double &op2);
	virtual hVarBase* operator/(const std::complex<float> &op2);
	virtual hVarBase* operator/(const std::complex<double> &op2);
	virtual hVarBase* operator/(const std::complex<long double> &op2);
	virtual hVarBase* operator%(const bool &op2);
	virtual hVarBase* operator%(const unsigned char &op2);
	virtual hVarBase* operator%(const signed char &op2);
	virtual hVarBase* operator%(const unsigned short &op2);
	virtual hVarBase* operator%(const signed short &op2);
	virtual hVarBase* operator%(const unsigned long &op2);
	virtual hVarBase* operator%(const signed long &op2);
	virtual hVarBase* operator%(const float &op2);
	virtual hVarBase* operator%(const double &op2);
	virtual hVarBase* operator%(const long double &op2);
	virtual hVarBase* operator%(const std::complex<float> &op2);
	virtual hVarBase* operator%(const std::complex<double> &op2);
	virtual hVarBase* operator%(const std::complex<long double> &op2);

	virtual hVarBase* operator=(const bool &op2);
	virtual hVarBase* operator=(const unsigned char &op2);
	virtual hVarBase* operator=(const signed char &op2);
	virtual hVarBase* operator=(const unsigned short &op2);
	virtual hVarBase* operator=(const signed short &op2);
	virtual hVarBase* operator=(const unsigned long &op2);
	virtual hVarBase* operator=(const signed long &op2);
	virtual hVarBase* operator=(const float &op2);
	virtual hVarBase* operator=(const double &op2);
	virtual hVarBase* operator=(const long double &op2);
	virtual hVarBase* operator=(const std::complex<float> &op2);
	virtual hVarBase* operator=(const std::complex<double> &op2);
	virtual hVarBase* operator=(const std::complex<long double> &op2);

	virtual hVarBase* operator<(const bool &op2);
	virtual hVarBase* operator<(const unsigned char &op2);
	virtual hVarBase* operator<(const signed char &op2);
	virtual hVarBase* operator<(const unsigned short &op2);
	virtual hVarBase* operator<(const signed short &op2);
	virtual hVarBase* operator<(const unsigned long &op2);
	virtual hVarBase* operator<(const signed long &op2);
	virtual hVarBase* operator<(const float &op2);
	virtual hVarBase* operator<(const double &op2);
	virtual hVarBase* operator<(const long double &op2);
	virtual hVarBase* operator<(const std::complex<float> &op2);
	virtual hVarBase* operator<(const std::complex<double> &op2);
	virtual hVarBase* operator<(const std::complex<long double> &op2);
	virtual hVarBase* operator<=(const bool &op2);
	virtual hVarBase* operator<=(const unsigned char &op2);
	virtual hVarBase* operator<=(const signed char &op2);
	virtual hVarBase* operator<=(const unsigned short &op2);
	virtual hVarBase* operator<=(const signed short &op2);
	virtual hVarBase* operator<=(const unsigned long &op2);
	virtual hVarBase* operator<=(const signed long &op2);
	virtual hVarBase* operator<=(const float &op2);
	virtual hVarBase* operator<=(const double &op2);
	virtual hVarBase* operator<=(const long double &op2);
	virtual hVarBase* operator<=(const std::complex<float> &op2);
	virtual hVarBase* operator<=(const std::complex<double> &op2);
	virtual hVarBase* operator<=(const std::complex<long double> &op2);
	virtual hVarBase* operator>(const bool &op2);
	virtual hVarBase* operator>(const unsigned char &op2);
	virtual hVarBase* operator>(const signed char &op2);
	virtual hVarBase* operator>(const unsigned short &op2);
	virtual hVarBase* operator>(const signed short &op2);
	virtual hVarBase* operator>(const unsigned long &op2);
	virtual hVarBase* operator>(const signed long &op2);
	virtual hVarBase* operator>(const float &op2);
	virtual hVarBase* operator>(const double &op2);
	virtual hVarBase* operator>(const long double &op2);
	virtual hVarBase* operator>(const std::complex<float> &op2);
	virtual hVarBase* operator>(const std::complex<double> &op2);
	virtual hVarBase* operator>(const std::complex<long double> &op2);
	virtual hVarBase* operator>=(const bool &op2);
	virtual hVarBase* operator>=(const unsigned char &op2);
	virtual hVarBase* operator>=(const signed char &op2);
	virtual hVarBase* operator>=(const unsigned short &op2);
	virtual hVarBase* operator>=(const signed short &op2);
	virtual hVarBase* operator>=(const unsigned long &op2);
	virtual hVarBase* operator>=(const signed long &op2);
	virtual hVarBase* operator>=(const float &op2);
	virtual hVarBase* operator>=(const double &op2);
	virtual hVarBase* operator>=(const long double &op2);
	virtual hVarBase* operator>=(const std::complex<float> &op2);
	virtual hVarBase* operator>=(const std::complex<double> &op2);
	virtual hVarBase* operator>=(const std::complex<long double> &op2);

	virtual hVarBase* operator&&(const bool &op2);
	virtual hVarBase* operator&&(const unsigned char &op2);
	virtual hVarBase* operator&&(const signed char &op2);
	virtual hVarBase* operator&&(const unsigned short &op2);
	virtual hVarBase* operator&&(const signed short &op2);
	virtual hVarBase* operator&&(const unsigned long &op2);
	virtual hVarBase* operator&&(const signed long &op2);
	virtual hVarBase* operator&&(const float &op2);
	virtual hVarBase* operator&&(const double &op2);
	virtual hVarBase* operator&&(const long double &op2);
	virtual hVarBase* operator&&(const std::complex<float> &op2);
	virtual hVarBase* operator&&(const std::complex<double> &op2);
	virtual hVarBase* operator&&(const std::complex<long double> &op2);
	virtual hVarBase* operator||(const bool &op2);
	virtual hVarBase* operator||(const unsigned char &op2);
	virtual hVarBase* operator||(const signed char &op2);
	virtual hVarBase* operator||(const unsigned short &op2);
	virtual hVarBase* operator||(const signed short &op2);
	virtual hVarBase* operator||(const unsigned long &op2);
	virtual hVarBase* operator||(const signed long &op2);
	virtual hVarBase* operator||(const float &op2);
	virtual hVarBase* operator||(const double &op2);
	virtual hVarBase* operator||(const long double &op2);
	virtual hVarBase* operator||(const std::complex<float> &op2);
	virtual hVarBase* operator||(const std::complex<double> &op2);
	virtual hVarBase* operator||(const std::complex<long double> &op2);
	virtual hVarBase* operator==(const bool &op2);
	virtual hVarBase* operator==(const unsigned char &op2);
	virtual hVarBase* operator==(const signed char &op2);
	virtual hVarBase* operator==(const unsigned short &op2);
	virtual hVarBase* operator==(const signed short &op2);
	virtual hVarBase* operator==(const unsigned long &op2);
	virtual hVarBase* operator==(const signed long &op2);
	virtual hVarBase* operator==(const float &op2);
	virtual hVarBase* operator==(const double &op2);
	virtual hVarBase* operator==(const long double &op2);
	virtual hVarBase* operator==(const std::complex<float> &op2);
	virtual hVarBase* operator==(const std::complex<double> &op2);
	virtual hVarBase* operator==(const std::complex<long double> &op2);
	virtual hVarBase* operator!=(const bool &op2);
	virtual hVarBase* operator!=(const unsigned char &op2);
	virtual hVarBase* operator!=(const signed char &op2);
	virtual hVarBase* operator!=(const unsigned short &op2);
	virtual hVarBase* operator!=(const signed short &op2);
	virtual hVarBase* operator!=(const unsigned long &op2);
	virtual hVarBase* operator!=(const signed long &op2);
	virtual hVarBase* operator!=(const float &op2);
	virtual hVarBase* operator!=(const double &op2);
	virtual hVarBase* operator!=(const long double &op2);
	virtual hVarBase* operator!=(const std::complex<float> &op2);
	virtual hVarBase* operator!=(const std::complex<double> &op2);
	virtual hVarBase* operator!=(const std::complex<long double> &op2);

	virtual hVarBase* operator<<(const bool &op2);
	virtual hVarBase* operator<<(const unsigned char &op2);
	virtual hVarBase* operator<<(const signed char &op2);
	virtual hVarBase* operator<<(const unsigned short &op2);
	virtual hVarBase* operator<<(const signed short &op2);
	virtual hVarBase* operator<<(const unsigned long &op2);
	virtual hVarBase* operator<<(const signed long &op2);
	virtual hVarBase* operator<<(const float &op2);
	virtual hVarBase* operator<<(const double &op2);
	virtual hVarBase* operator<<(const long double &op2);
	virtual hVarBase* operator<<(const std::complex<float> &op2);
	virtual hVarBase* operator<<(const std::complex<double> &op2);
	virtual hVarBase* operator<<(const std::complex<long double> &op2);
	virtual hVarBase* operator>>(const bool &op2);
	virtual hVarBase* operator>>(const unsigned char &op2);
	virtual hVarBase* operator>>(const signed char &op2);
	virtual hVarBase* operator>>(const unsigned short &op2);
	virtual hVarBase* operator>>(const signed short &op2);
	virtual hVarBase* operator>>(const unsigned long &op2);
	virtual hVarBase* operator>>(const signed long &op2);
	virtual hVarBase* operator>>(const float &op2);
	virtual hVarBase* operator>>(const double &op2);
	virtual hVarBase* operator>>(const long double &op2);
	virtual hVarBase* operator>>(const std::complex<float> &op2);
	virtual hVarBase* operator>>(const std::complex<double> &op2);
	virtual hVarBase* operator>>(const std::complex<long double> &op2);
	virtual hVarBase* operator&(const bool &op2);
	virtual hVarBase* operator&(const unsigned char &op2);
	virtual hVarBase* operator&(const signed char &op2);
	virtual hVarBase* operator&(const unsigned short &op2);
	virtual hVarBase* operator&(const signed short &op2);
	virtual hVarBase* operator&(const unsigned long &op2);
	virtual hVarBase* operator&(const signed long &op2);
	virtual hVarBase* operator&(const float &op2);
	virtual hVarBase* operator&(const double &op2);
	virtual hVarBase* operator&(const long double &op2);
	virtual hVarBase* operator&(const std::complex<float> &op2);
	virtual hVarBase* operator&(const std::complex<double> &op2);
	virtual hVarBase* operator&(const std::complex<long double> &op2);
	virtual hVarBase* operator|(const bool &op2);
	virtual hVarBase* operator|(const unsigned char &op2);
	virtual hVarBase* operator|(const signed char &op2);
	virtual hVarBase* operator|(const unsigned short &op2);
	virtual hVarBase* operator|(const signed short &op2);
	virtual hVarBase* operator|(const unsigned long &op2);
	virtual hVarBase* operator|(const signed long &op2);
	virtual hVarBase* operator|(const float &op2);
	virtual hVarBase* operator|(const double &op2);
	virtual hVarBase* operator|(const long double &op2);
	virtual hVarBase* operator|(const std::complex<float> &op2);
	virtual hVarBase* operator|(const std::complex<double> &op2);
	virtual hVarBase* operator|(const std::complex<long double> &op2);
	virtual hVarBase* operator^(const bool &op2);
	virtual hVarBase* operator^(const unsigned char &op2);
	virtual hVarBase* operator^(const signed char &op2);
	virtual hVarBase* operator^(const unsigned short &op2);
	virtual hVarBase* operator^(const signed short &op2);
	virtual hVarBase* operator^(const unsigned long &op2);
	virtual hVarBase* operator^(const signed long &op2);
	virtual hVarBase* operator^(const float &op2);
	virtual hVarBase* operator^(const double &op2);
	virtual hVarBase* operator^(const long double &op2);
	virtual hVarBase* operator^(const std::complex<float> &op2);
	virtual hVarBase* operator^(const std::complex<double> &op2);
	virtual hVarBase* operator^(const std::complex<long double> &op2);

	virtual hVarBase* operator+(const hVarBase* op2)=0;
	virtual hVarBase* operator-(const hVarBase* op2)=0;
	virtual hVarBase* operator*(const hVarBase* op2)=0;
	virtual hVarBase* operator/(const hVarBase* op2)=0;
	virtual hVarBase* operator%(const hVarBase* op2)=0;

//compiler is buggy. hVarBase * gets mixed up with dataType &
//	virtual hVarBase* operator=(const hVarBase* op2)=0;

	virtual hVarBase* operator<(const hVarBase* op2)=0;
	virtual hVarBase* operator>(const hVarBase* op2)=0;
	virtual hVarBase* operator<=(const hVarBase* op2)=0;
	virtual hVarBase* operator>=(const hVarBase* op2)=0;

	virtual hVarBase* operator!(void)=0;
	virtual hVarBase* operator&&(const hVarBase* op2)=0;
	virtual hVarBase* operator||(const hVarBase* op2)=0;
	virtual hVarBase* operator==(const hVarBase* op2)=0;
	virtual hVarBase* operator!=(const hVarBase* op2)=0;

	virtual hVarBase* operator~(void)=0;

	virtual hVarBase* operator<<(const hVarBase* op2)=0;
	virtual hVarBase* operator>>(const hVarBase* op2)=0;
	virtual hVarBase* operator&(const hVarBase* op2)=0;
	virtual hVarBase* operator|(const hVarBase* op2)=0;
	virtual hVarBase* operator^(const hVarBase* op2)=0;

	virtual hVarBase* operator++(void)=0;//++a
	virtual hVarBase* operator--(void)=0;

	virtual hVarBase* operator[](unsigned int index)=0;
	virtual hVarBase* operator[](hVarBase* index)=0;

	virtual hVarBase* operator()(const hVarBase* op2)=0;
	virtual hVarBase* operator,(const hVarBase* op2)=0;

	virtual hVarBase* traverse(float (*function)(float)){};
	virtual hVarBase* traverse(double (*function)(double)){};

	virtual void print(std::ostream& output)=0;
};

#endif
